// ======================================================================
// ST_INC_AI  - AI-related generic utility functions
// ======================================================================
// Last changed: 2006-07-17
// ----------------------------------------------------------------------

// ======================================================================
// FUNCTION PROTOTYPES
// ----------------------------------------------------------------------

void   DebugMessage(string sText, int bTimestamp=TRUE);
void   DebugMessageParty(string sMessage);
void   ST_SetLastAttackTarget(object oTarget);
void   ST_AssignHLPowers(int bBlock=FALSE);

int    ST_GetIsPowerOrBetterEffect(effect eEff, int nSpellID);
int    ST_GetHasSpellEffectOrRelated(int nSpellID, object oTarget);
int    ST_ShouldUseDisablingPower(talent tUse, object oTarget);
int    ST_GetIsShielded(object oTarget=OBJECT_SELF);
int    ST_GetTrapInPathTo(object oTarget);
int    ST_IsPartyMemberOrAssociate(object oTarget = OBJECT_SELF);
int    ST_GetIsEnraged(object oTarget);
int    ST_GetDoorInPathTo(object oTarget);
int    ST_IsValidAttackTarget(object oTarget, object oLeader, float fDist);
int    ST_GetHasHLSpell(int nSpell, object oTarget=OBJECT_SELF, int bBuff=FALSE);
int    ST_GetIsForceUser(object oTarget=OBJECT_SELF);
int    ST_TalentUseRage();
int    ST_GetPerformsCombatAction(object oTarget);
int    ST_ActionUseTalent(talent tUse, object oTarget, int bSetTarget=TRUE, int bClearActions=TRUE);
int    ST_RageFeatHack(int nType);
int    ST_GetIsAITargetValid(object oTarget);
int    ST_GetEnemyHPTotal();
int    ST_GetIsHelpless(object oTarget);
int    ST_GetHasLightsaber(object oTarget=OBJECT_SELF);
int    ST_GetIsCombatFeat(int nFeat);
int    ST_GetHasClearPathTo(object oTarget);
int    ST_TalentBuffUseStims(object oUser=OBJECT_SELF);
int    ST_GetHasStimEffect(int nSpell, object oTarget=OBJECT_SELF);
int    ST_HasOnlyDroidShockArm();

float  ST_GetAttackRange(object oAttacker = OBJECT_SELF);
			
object ST_GetTargetInRange(float fDist, object oTarget, object oIntruder=OBJECT_INVALID);
object ST_GetLastAttackTarget();
object ST_SelectAttackTarget(object oIntruder=OBJECT_INVALID);
object ST_GetNearestEnemy(object oSource = OBJECT_SELF);

string IntToStringPad(int nVal, int nDigits);

talent ST_SubstPowerIfPresent(talent tUse, int nPower1, int nPower2 = -1);
talent ST_SubstFeatIfPresent(talent tUse, int nFeat1, int nFeat2 = -1);
talent ST_GetCorrectForcePower(talent tUse);
talent ST_GetCorrectCombatFeat(talent tUse);
talent ST_GetCorrectForceOrFeat(talent tUse);
talent ST_GetRandomProjectileTalent();


// ======================================================================
// IMPLEMENTED FUNCTIONS
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// DebugMessage()
// 		Put a timestamped debug-message string in the Feedback section of
// 		the game logs screen.
// ----------------------------------------------------------------------
void DebugMessage(string sText, int bTimestamp=TRUE) {
	string sTimestamp = "";
	
	if (bTimestamp) {
		int nHour = GetTimeHour();
		int nMin  = GetTimeMinute(); // ST: Ingame-time! There are 2 min/hour.
		int nSec  = GetTimeSecond();
		int nMSec = GetTimeMillisecond();
		
		sTimestamp = (nHour < 10 ? "0" : "") + IntToString(nHour)
			 + ":" + (nMin  < 10 ? "0" : "") + IntToString(nMin)
			 + ":" + (nSec  < 10 ? "0" : "") + IntToString(nSec)
			 + ":" + GetSubString((nMSec  < 10 ? "0" : "") + IntToString(nMSec), 0, 2);	
			 
		sTimestamp += " - ";	
	}
	
	SendMessageToPC(GetPartyLeader(), sTimestamp + "DEBUG[" + GetName(OBJECT_SELF) + "] " + sText);
}


// ----------------------------------------------------------------------
// DebugMessageParty()
// 		Post a debug-message to the Feedback screen if the calling object
// 		is a party member running on AI scripts (ie not selected member).
// ----------------------------------------------------------------------
void DebugMessageParty(string sMessage) {
	if (IsObjectPartyMember(OBJECT_SELF) && (GetPartyMemberByIndex(0) != OBJECT_SELF))
		DebugMessage(sMessage);
}


// ----------------------------------------------------------------------
// ST_GetIsPowerOrBetterEffect()
// 		Check if the specified effect belongs to the force power specified
//     	by nSpellID, or a better power related to the same family of
//     	powers as the specified one.
// ----------------------------------------------------------------------
int ST_GetIsPowerOrBetterEffect(effect eEff, int nSpellID) {

    int nRelated1 = -1;
    int nRelated2 = -1;
    int nRelated3 = -1;
    int nRelated4 = -1;
    int nRelated5 = -1;
    
    int nEffId = GetEffectSpellId(eEff);
    
    switch ( nSpellID )
    {
        case FORCE_POWER_SLOW:
            nRelated1 = FORCE_POWER_AFFLICTION;
            nRelated2 = FORCE_POWER_PLAGUE;
            break;
        case FORCE_POWER_AFFLICTION:
            nRelated1 = FORCE_POWER_PLAGUE;
            break;
        case FORCE_POWER_FEAR:
        	nRelated1 = FORCE_POWER_HORROR;
        	nRelated2 = FORCE_POWER_INSANITY;
        	break;
        case FORCE_POWER_HORROR:
        	nRelated1 = FORCE_POWER_INSANITY;
        	break;
        case FORCE_POWER_FORCE_AURA:
            nRelated1 = FORCE_POWER_FORCE_SHIELD;
            nRelated2 = FORCE_POWER_FORCE_ARMOR;
            break;
        case FORCE_POWER_FORCE_SHIELD:
            nRelated1 = FORCE_POWER_FORCE_ARMOR;
            break;
        case FORCE_POWER_RESIST_FORCE:
            nRelated1 = FORCE_POWER_FORCE_IMMUNITY;
            break;
        case FORCE_POWER_FORCE_MIND:
            nRelated1 = FORCE_POWER_KNIGHT_MIND;
            nRelated2 = FORCE_POWER_MIND_MASTERY;
            break;
        case FORCE_POWER_KNIGHT_MIND:
            nRelated1 = FORCE_POWER_MIND_MASTERY;
            break;
        case FORCE_POWER_RESIST_POISON_DISEASE_SONIC:
            nRelated1 = FORCE_POWER_RESIST_COLD_HEAT_ENERGY;
            nRelated2 = FORCE_POWER_MASTER_ENERGY_RESISTANCE;
            break;
        case FORCE_POWER_RESIST_COLD_HEAT_ENERGY:
            nRelated1 = FORCE_POWER_MASTER_ENERGY_RESISTANCE;
            break;
        case FORCE_POWER_STUN:
            nRelated1 = FORCE_POWER_HOLD;
            nRelated2 = FORCE_POWER_SLEEP;
            break;
        case FORCE_POWER_HOLD:
            nRelated1 = FORCE_POWER_SLEEP;
            break;
        case FORCE_POWER_SPEED_BURST:
            nRelated1 = FORCE_POWER_KNIGHT_SPEED;
            nRelated2 = FORCE_POWER_SPEED_MASTERY;
            break;
        case FORCE_POWER_KNIGHT_SPEED:
            nRelated1 = FORCE_POWER_SPEED_MASTERY;
            break;
        case FORCE_POWER_FORCE_BARRIER:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_BARRIER;
            nRelated2 = FORCE_POWER_MASTER_FORCE_BARRIER;
            break;
        case FORCE_POWER_IMPROVED_FORCE_BARRIER:
            nRelated1 = FORCE_POWER_MASTER_FORCE_BARRIER;
            break;
        case FORCE_POWER_BATTLE_MEDITATION_PC:
            nRelated1 = FORCE_POWER_IMPROVED_BATTLE_MEDITATION_PC;
            nRelated2 = FORCE_POWER_MASTER_BATTLE_MEDITATION_PC;
            break;
        case FORCE_POWER_IMPROVED_BATTLE_MEDITATION_PC:
            nRelated1 = FORCE_POWER_MASTER_BATTLE_MEDITATION_PC;
            break;
        case FORCE_POWER_BAT_MED_ENEMY:
            nRelated1 = FORCE_POWER_IMP_BAT_MED_ENEMY;
            nRelated2 = FORCE_POWER_MAS_BAT_MED_ENEMY;
            break;
        case FORCE_POWER_IMP_BAT_MED_ENEMY:
            nRelated1 = FORCE_POWER_MAS_BAT_MED_ENEMY;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_I:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_II;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_III;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated4 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated5 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_II:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_III;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated4 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_III:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_IV:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_V:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_FORCE_BODY:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_BODY;
            nRelated2 = FORCE_POWER_MASTER_FORCE_BODY;
            break;
        case FORCE_POWER_IMPROVED_FORCE_BODY:
            nRelated1 = FORCE_POWER_MASTER_FORCE_BODY;
            break;
        case FORCE_POWER_FORCE_SCREAM:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_SCREAM;
            nRelated2 = FORCE_POWER_MASTER_FORCE_SCREAM;
            break;
        case FORCE_POWER_IMPROVED_FORCE_SCREAM:
            nRelated1 = FORCE_POWER_MASTER_FORCE_SCREAM;
            break;
        case FORCE_POWER_FURY:
            nRelated1 = FORCE_POWER_IMPROVED_FURY;
            nRelated2 = FORCE_POWER_MASTER_FURY;
            break;
        case FORCE_POWER_IMPROVED_FURY:
            nRelated1 = FORCE_POWER_MASTER_FURY;
            break;
        case FORCE_POWER_WOOKIEE_RAGE_I:
            nRelated1 = FORCE_POWER_WOOKIEE_RAGE_II;
            nRelated2 = FORCE_POWER_WOOKIEE_RAGE_III;
            break;
        case FORCE_POWER_WOOKIEE_RAGE_II:
            nRelated1 = FORCE_POWER_WOOKIEE_RAGE_III;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_I:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_II;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_III;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated4 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated5 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_II:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_III;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated4 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_III:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_IV:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_V:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_MIND_TRICK:
            nRelated1 = FORCE_POWER_CONFUSION;
            break;
        case FORCE_POWER_BEAST_TRICK:
            nRelated1 = FORCE_POWER_BEAST_CONFUSION;
            break;
        case FORCE_POWER_DROID_TRICK:
            nRelated1 = FORCE_POWER_DROID_CONFUSION;
            break;
        case JD_BLIND:					      
            nRelated2 = JD_BLIND_IMP;		
            nRelated3 = JD_BLIND_MSTR;		
            break;
        case JD_BLIND_IMP:					
            nRelated2 = JD_BLIND_MSTR;		
            break;
        case JD_DECEIVE:					
            nRelated2 = JD_DECEIVE_IMP;		
            nRelated3 = JD_DECEIVE_MSTR;		
            break;
        case JD_DECEIVE_IMP:					
            nRelated2 = JD_DECEIVE_MSTR;		
            break;
        case JD_LEECH:   					
            nRelated2 = JD_LEECH_IMP;		
            nRelated3 = JD_LEECH_MSTR;		
            break;
        case JD_LEECH_IMP:					
            nRelated3 = JD_LEECH_MSTR;		
            break;
        case JD_ENERGY: 					
            nRelated2 = JD_ENERGY_IMP;		
            nRelated3 = JD_ENERGY_MSTR;		
            break;
        case JD_ENERGY_IMP:					
            nRelated3 = JD_ENERGY_MSTR;		
            break;
        case JD_STATIC:  					
            nRelated2 = JD_STATIC_IMP;		
            nRelated3 = JD_STATIC_MSTR;		
            break;
        case JD_STATIC_IMP:					
            nRelated3 = JD_STATIC_MSTR;		
            break;
        case JD_CONTAGION:					
            nRelated2 = JD_CONTAGION_IMP;	
            nRelated3 =JD_CONTAGION_MSTR;	
            break;
        case JD_CONTAGION_IMP:				
            nRelated3 = JD_CONTAGION_MSTR;	
            break;
        case JD_BOMBARD:					
            nRelated2 = JD_BOMBARD_IMP;		
            nRelated3 = JD_BOMBARD_MSTR;		
            break;
        case JD_BOMBARD_IMP:					
            nRelated3 = JD_BOMBARD_MSTR;		
            break;
        case JD_DESTRUCT:					
            nRelated2 = JD_DESTRUCT_IMP;		
            nRelated3 = JD_DESTRUCT_MSTR;	
            break;
        case JD_DESTRUCT_IMP:					
            nRelated3 = JD_DESTRUCT_MSTR;	
            break;
        case JD_INFERNO:					
            nRelated2 = JD_INFERNO_IMP;		
            nRelated3 = JD_INFERNO_MSTR;		
            break;
        case JD_INFERNO_IMP:					
            nRelated3 = JD_INFERNO_MSTR;		
            break;
        case JD_REGEN:  					
            nRelated2 = JD_REGEN_IMP;		
            nRelated3 = JD_REGEN_MSTR;		
            break;
        case JD_REGEN_IMP:					
            nRelated3 = JD_REGEN_MSTR;		
            break;
        case JD_PROTECT:					
            nRelated2 = JD_PROTECT_IMP;		
            nRelated3 = JD_PROTECT_MSTR;		
            break;
        case JD_PROTECT_IMP:					
            nRelated3 = JD_PROTECT_MSTR;		
            break;
        case JD_GRIP:	       				
            nRelated2 = JD_GRIP_IMP;		
            nRelated3 = JD_GRIP_MSTR;		
            break;
        case JD_GRIP_IMP:					
            nRelated3 = JD_GRIP_MSTR;		
            break;
        case JD_OMNI:	      				
            nRelated2 = JD_OMNI_IMP;		
            nRelated3 = JD_OMNI_MSTR;		
            break;
        case JD_OMNI_IMP:					
            nRelated3 = JD_OMNI_MSTR;		
            break;
        case JD_FREEZE:  					
            nRelated2 = JD_FREEZE_IMP;		
            nRelated3 = JD_FREEZE_MSTR;		
            break;
        case JD_FREEZE_IMP:					
            nRelated3 = JD_FREEZE_MSTR;		
            break;
        case JD_HOLD:    					
            nRelated2 = JD_HOLD_IMP;		
            nRelated3 = JD_HOLD_MSTR;		
            break;
        case JD_HOLD_IMP:					
            nRelated3 = JD_HOLD_MSTR;		
            break;
        case JD_SAPWILL:					
            nRelated2 = JD_SAPWILL_IMP;		
            nRelated3 = JD_SAPWILL_MSTR;		
            break;
        case JD_SAPWILL_IMP:					
            nRelated3 = JD_SAPWILL_MSTR;		
            break;
        case JD_WEAKEN:	      				
            nRelated2 = JD_WEAKEN_IMP;		
            nRelated3 = JD_WEAKEN_MSTR;		
            break;
        case JD_WEAKEN_IMP:					
            nRelated3 = JD_WEAKEN_MSTR;		
            break;
        case JD_STRIKE:		      			
            nRelated2 = JD_STRIKE_IMP;		
            nRelated3 = JD_STRIKE_MSTR;		
            break;
        case JD_STRIKE_IMP:					
            nRelated3 = JD_STRIKE_MSTR;		
            break;
        case JD_HURRICANE:					
            nRelated2 = JD_HURRICANE_IMP;	
            nRelated3 = JD_HURRICANE_MSTR;	
            break;
        case JD_HURRICANE_IMP:				
            nRelated3 = JD_HURRICANE_MSTR;	
            break;
        case JD_DEMO:   					
            nRelated2 = JD_IMP_DEMO;		
            nRelated3 = JD_MSTR_DEMO;		
            break;
        case JD_IMP_DEMO:					
            nRelated3 = JD_MSTR_DEMO;		
            break;
        case JD_AWARE:	       				
            nRelated2 = JD_IMP_AWARE;		
            nRelated3 = JD_MSTR_AWARE;		
            break;
        case JD_IMP_AWARE:					
            nRelated3 = JD_MSTR_AWARE;		
            break;
        case JD_COMP:	       				
            nRelated2 = JD_IMP_COMP;		
            nRelated3 = JD_MSTR_COMP;		
            break;
        case JD_IMP_COMP:					
            nRelated3 = JD_MSTR_COMP;		
            break;
        case JD_PERS:    					
            nRelated2 = JD_IMP_PERS;		
            nRelated3 = JD_MSTR_PERS;		
            break;	
        case JD_IMP_PERS:					
            nRelated3 = JD_MSTR_PERS;		
            break;
        case JD_REPAIR:	      				
            nRelated2 = JD_IMP_REPAIR;		
            nRelated3 = JD_MSTR_REPAIR;		
            break;
        case JD_IMP_REPAIR:					
            nRelated3 = JD_MSTR_REPAIR;		
            break;
        case JD_SECURITY:					
            nRelated2 = JD_IMP_SECURITY;		
            nRelated3 = JD_MSTR_SECURITY;	
            break;
        case JD_IMP_SECURITY:					
            nRelated3 = JD_MSTR_SECURITY;	
            break;
        case JD_STEALTH:					
            nRelated2 = JD_IMP_STEALTH;		
            nRelated3 = JD_MSTR_STEALTH;		
            break;
        case JD_IMP_STEALTH:					
            nRelated3 = JD_MSTR_STEALTH;		
            break;
        case JD_TREAT_INJURY:					
            nRelated2 = JD_IMP_TREAT_INJURY;	
            nRelated3 = JD_MSTR_TREAT_INJURY;	
            break;
        case JD_IMP_TREAT_INJURY:				
            nRelated3 = JD_MSTR_TREAT_INJURY;  
            break;
        case JD_RECK:    					
            nRelated2 = JD_RECK_IMP;		
            nRelated3 = JD_RECK_MSTR;		
            break;
        case JD_RECK_IMP:					
            nRelated3 = JD_RECK_MSTR;		
            break;
        case JD_RAGE:	       				
            nRelated2 = JD_RAGE_IMP;		
            nRelated3 = JD_RAGE_MSTR;		
            break;
        case JD_RAGE_IMP:					
            nRelated3 = JD_RAGE_MSTR;		
            break;
        case JD_HEAL_FORCE_MINOR:	       		
            nRelated2 = JD_HEAL_FORCE_AVERAGE;	
            nRelated3 = JD_HEAL_FORCE_STRONG;	
            break;
        case JD_HEAL_FORCE_AVERAGE:				
            nRelated3 = JD_HEAL_FORCE_STRONG;	
            break;
        default:
        	// ST: Power has no related ones, just check if for the specified power.
            return (nEffId == nSpellID);
            break;
    }

    if( 	(nEffId == nSpellID)
    	|| 	(nEffId == nRelated1)
        || 	(nEffId == nRelated2)
        || 	(nEffId == nRelated3)
        || 	(nEffId == nRelated4)
        || 	(nEffId == nRelated5) )
    {
        return TRUE;
    }

    return FALSE;
}


// ----------------------------------------------------------------------
// ST_GetHasSpellEffectOrRelated()
// 		Check if the specified target has an effect from the force power
//      in nSpellID, or from a better power in the same family.
// ----------------------------------------------------------------------
int ST_GetHasSpellEffectOrRelated(int nSpellID, object oTarget) {

    int nRelated1 = -1;
    int nRelated2 = -1;
    int nRelated3 = -1;
    int nRelated4 = -1;
    int nRelated5 = -1;
    
    switch ( nSpellID )
    {
        case FORCE_POWER_SLOW:
            nRelated1 = FORCE_POWER_AFFLICTION;
            nRelated2 = FORCE_POWER_PLAGUE;
            break;
        case FORCE_POWER_AFFLICTION:
            nRelated1 = FORCE_POWER_PLAGUE;
            break;
        case FORCE_POWER_FEAR:
        	nRelated1 = FORCE_POWER_HORROR;
        	nRelated2 = FORCE_POWER_INSANITY;
        	break;
        case FORCE_POWER_HORROR:
        	nRelated1 = FORCE_POWER_INSANITY;
        	break;
        case FORCE_POWER_FORCE_AURA:
            nRelated1 = FORCE_POWER_FORCE_SHIELD;
            nRelated2 = FORCE_POWER_FORCE_ARMOR;
            break;
        case FORCE_POWER_FORCE_SHIELD:
            nRelated1 = FORCE_POWER_FORCE_ARMOR;
            break;
        case FORCE_POWER_RESIST_FORCE:
            nRelated1 = FORCE_POWER_FORCE_IMMUNITY;
            break;
        case FORCE_POWER_FORCE_MIND:
            nRelated1 = FORCE_POWER_KNIGHT_MIND;
            nRelated2 = FORCE_POWER_MIND_MASTERY;
            break;
        case FORCE_POWER_KNIGHT_MIND:
            nRelated1 = FORCE_POWER_MIND_MASTERY;
            break;
        case FORCE_POWER_RESIST_POISON_DISEASE_SONIC:
            nRelated1 = FORCE_POWER_RESIST_COLD_HEAT_ENERGY;
            nRelated2 = FORCE_POWER_MASTER_ENERGY_RESISTANCE;
            break;
        case FORCE_POWER_RESIST_COLD_HEAT_ENERGY:
            nRelated1 = FORCE_POWER_MASTER_ENERGY_RESISTANCE;
            break;
        case FORCE_POWER_STUN:
            nRelated1 = FORCE_POWER_HOLD;
            nRelated2 = FORCE_POWER_SLEEP;
            break;
        case FORCE_POWER_HOLD:
            nRelated1 = FORCE_POWER_SLEEP;
            break;
        case FORCE_POWER_SPEED_BURST:
            nRelated1 = FORCE_POWER_KNIGHT_SPEED;
            nRelated2 = FORCE_POWER_SPEED_MASTERY;
            break;
        case FORCE_POWER_KNIGHT_SPEED:
            nRelated1 = FORCE_POWER_SPEED_MASTERY;
            break;
        case FORCE_POWER_FORCE_BARRIER:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_BARRIER;
            nRelated2 = FORCE_POWER_MASTER_FORCE_BARRIER;
            break;
        case FORCE_POWER_IMPROVED_FORCE_BARRIER:
            nRelated1 = FORCE_POWER_MASTER_FORCE_BARRIER;
            break;
        case FORCE_POWER_BATTLE_MEDITATION_PC:
            nRelated1 = FORCE_POWER_IMPROVED_BATTLE_MEDITATION_PC;
            nRelated2 = FORCE_POWER_MASTER_BATTLE_MEDITATION_PC;
            break;
        case FORCE_POWER_IMPROVED_BATTLE_MEDITATION_PC:
            nRelated1 = FORCE_POWER_MASTER_BATTLE_MEDITATION_PC;
            break;
        case FORCE_POWER_BAT_MED_ENEMY:
            nRelated1 = FORCE_POWER_IMP_BAT_MED_ENEMY;
            nRelated2 = FORCE_POWER_MAS_BAT_MED_ENEMY;
            break;
        case FORCE_POWER_IMP_BAT_MED_ENEMY:
            nRelated1 = FORCE_POWER_MAS_BAT_MED_ENEMY;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_I:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_II;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_III;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated4 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated5 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_II:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_III;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated4 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_III:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_IV;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated3 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_IV:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_V;
            nRelated2 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_CRUSH_OPPOSITION_V:
            nRelated1 = FORCE_POWER_CRUSH_OPPOSITION_VI;
            break;
        case FORCE_POWER_FORCE_BODY:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_BODY;
            nRelated2 = FORCE_POWER_MASTER_FORCE_BODY;
            break;
        case FORCE_POWER_IMPROVED_FORCE_BODY:
            nRelated1 = FORCE_POWER_MASTER_FORCE_BODY;
            break;
        case FORCE_POWER_FORCE_SCREAM:
            nRelated1 = FORCE_POWER_IMPROVED_FORCE_SCREAM;
            nRelated2 = FORCE_POWER_MASTER_FORCE_SCREAM;
            break;
        case FORCE_POWER_IMPROVED_FORCE_SCREAM:
            nRelated1 = FORCE_POWER_MASTER_FORCE_SCREAM;
            break;
        case FORCE_POWER_FURY:
            nRelated1 = FORCE_POWER_IMPROVED_FURY;
            nRelated2 = FORCE_POWER_MASTER_FURY;
            break;
        case FORCE_POWER_IMPROVED_FURY:
            nRelated1 = FORCE_POWER_MASTER_FURY;
            break;
        case FORCE_POWER_WOOKIEE_RAGE_I:
            nRelated1 = FORCE_POWER_WOOKIEE_RAGE_II;
            nRelated2 = FORCE_POWER_WOOKIEE_RAGE_III;
            break;
        case FORCE_POWER_WOOKIEE_RAGE_II:
            nRelated1 = FORCE_POWER_WOOKIEE_RAGE_III;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_I:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_II;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_III;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated4 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated5 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_II:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_III;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated4 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_III:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_IV;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated3 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_IV:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_V;
            nRelated2 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_INSPIRE_FOLLOWERS_V:
            nRelated1 = FORCE_POWER_INSPIRE_FOLLOWERS_VI;
            break;
        case FORCE_POWER_MIND_TRICK:
            nRelated1 = FORCE_POWER_CONFUSION;
            break;
        case FORCE_POWER_BEAST_TRICK:
            nRelated1 = FORCE_POWER_BEAST_CONFUSION;
            break;
        case FORCE_POWER_DROID_TRICK:
            nRelated1 = FORCE_POWER_DROID_CONFUSION;
            break;
        case JD_BLIND:					      
            nRelated2 = JD_BLIND_IMP;		
            nRelated3 = JD_BLIND_MSTR;		
            break;
        case JD_BLIND_IMP:					
            nRelated2 = JD_BLIND_MSTR;		
            break;
        case JD_DECEIVE:					
            nRelated2 = JD_DECEIVE_IMP;		
            nRelated3 = JD_DECEIVE_MSTR;		
            break;
        case JD_DECEIVE_IMP:					
            nRelated2 = JD_DECEIVE_MSTR;		
            break;
        case JD_LEECH:   					
            nRelated2 = JD_LEECH_IMP;		
            nRelated3 = JD_LEECH_MSTR;		
            break;
        case JD_LEECH_IMP:					
            nRelated3 = JD_LEECH_MSTR;		
            break;
        case JD_ENERGY: 					
            nRelated2 = JD_ENERGY_IMP;		
            nRelated3 = JD_ENERGY_MSTR;		
            break;
        case JD_ENERGY_IMP:					
            nRelated3 = JD_ENERGY_MSTR;		
            break;
        case JD_STATIC:  					
            nRelated2 = JD_STATIC_IMP;		
            nRelated3 = JD_STATIC_MSTR;		
            break;
        case JD_STATIC_IMP:					
            nRelated3 = JD_STATIC_MSTR;		
            break;
        case JD_CONTAGION:					
            nRelated2 = JD_CONTAGION_IMP;	
            nRelated3 =JD_CONTAGION_MSTR;	
            break;
        case JD_CONTAGION_IMP:				
            nRelated3 = JD_CONTAGION_MSTR;	
            break;
        case JD_BOMBARD:					
            nRelated2 = JD_BOMBARD_IMP;		
            nRelated3 = JD_BOMBARD_MSTR;		
            break;
        case JD_BOMBARD_IMP:					
            nRelated3 = JD_BOMBARD_MSTR;		
            break;
        case JD_DESTRUCT:					
            nRelated2 = JD_DESTRUCT_IMP;		
            nRelated3 = JD_DESTRUCT_MSTR;	
            break;
        case JD_DESTRUCT_IMP:					
            nRelated3 = JD_DESTRUCT_MSTR;	
            break;
        case JD_INFERNO:					
            nRelated2 = JD_INFERNO_IMP;		
            nRelated3 = JD_INFERNO_MSTR;		
            break;
        case JD_INFERNO_IMP:					
            nRelated3 = JD_INFERNO_MSTR;		
            break;
        case JD_REGEN:  					
            nRelated2 = JD_REGEN_IMP;		
            nRelated3 = JD_REGEN_MSTR;		
            break;
        case JD_REGEN_IMP:					
            nRelated3 = JD_REGEN_MSTR;		
            break;
        case JD_PROTECT:					
            nRelated2 = JD_PROTECT_IMP;		
            nRelated3 = JD_PROTECT_MSTR;		
            break;
        case JD_PROTECT_IMP:					
            nRelated3 = JD_PROTECT_MSTR;		
            break;
        case JD_GRIP:	       				
            nRelated2 = JD_GRIP_IMP;		
            nRelated3 = JD_GRIP_MSTR;		
            break;
        case JD_GRIP_IMP:					
            nRelated3 = JD_GRIP_MSTR;		
            break;
        case JD_OMNI:	      				
            nRelated2 = JD_OMNI_IMP;		
            nRelated3 = JD_OMNI_MSTR;		
            break;
        case JD_OMNI_IMP:					
            nRelated3 = JD_OMNI_MSTR;		
            break;
        case JD_FREEZE:  					
            nRelated2 = JD_FREEZE_IMP;		
            nRelated3 = JD_FREEZE_MSTR;		
            break;
        case JD_FREEZE_IMP:					
            nRelated3 = JD_FREEZE_MSTR;		
            break;
        case JD_HOLD:    					
            nRelated2 = JD_HOLD_IMP;		
            nRelated3 = JD_HOLD_MSTR;		
            break;
        case JD_HOLD_IMP:					
            nRelated3 = JD_HOLD_MSTR;		
            break;
        case JD_SAPWILL:					
            nRelated2 = JD_SAPWILL_IMP;		
            nRelated3 = JD_SAPWILL_MSTR;		
            break;
        case JD_SAPWILL_IMP:					
            nRelated3 = JD_SAPWILL_MSTR;		
            break;
        case JD_WEAKEN:	      				
            nRelated2 = JD_WEAKEN_IMP;		
            nRelated3 = JD_WEAKEN_MSTR;		
            break;
        case JD_WEAKEN_IMP:					
            nRelated3 = JD_WEAKEN_MSTR;		
            break;
        case JD_STRIKE:		      			
            nRelated2 = JD_STRIKE_IMP;		
            nRelated3 = JD_STRIKE_MSTR;		
            break;
        case JD_STRIKE_IMP:					
            nRelated3 = JD_STRIKE_MSTR;		
            break;
        case JD_HURRICANE:					
            nRelated2 = JD_HURRICANE_IMP;	
            nRelated3 = JD_HURRICANE_MSTR;	
            break;
        case JD_HURRICANE_IMP:				
            nRelated3 = JD_HURRICANE_MSTR;	
            break;
        case JD_DEMO:   					
            nRelated2 = JD_IMP_DEMO;		
            nRelated3 = JD_MSTR_DEMO;		
            break;
        case JD_IMP_DEMO:					
            nRelated3 = JD_MSTR_DEMO;		
            break;
        case JD_AWARE:	       				
            nRelated2 = JD_IMP_AWARE;		
            nRelated3 = JD_MSTR_AWARE;		
            break;
        case JD_IMP_AWARE:					
            nRelated3 = JD_MSTR_AWARE;		
            break;
        case JD_COMP:	       				
            nRelated2 = JD_IMP_COMP;		
            nRelated3 = JD_MSTR_COMP;		
            break;
        case JD_IMP_COMP:					
            nRelated3 = JD_MSTR_COMP;		
            break;
        case JD_PERS:    					
            nRelated2 = JD_IMP_PERS;		
            nRelated3 = JD_MSTR_PERS;		
            break;	
        case JD_IMP_PERS:					
            nRelated3 = JD_MSTR_PERS;		
            break;
        case JD_REPAIR:	      				
            nRelated2 = JD_IMP_REPAIR;		
            nRelated3 = JD_MSTR_REPAIR;		
            break;
        case JD_IMP_REPAIR:					
            nRelated3 = JD_MSTR_REPAIR;		
            break;
        case JD_SECURITY:					
            nRelated2 = JD_IMP_SECURITY;		
            nRelated3 = JD_MSTR_SECURITY;	
            break;
        case JD_IMP_SECURITY:					
            nRelated3 = JD_MSTR_SECURITY;	
            break;
        case JD_STEALTH:					
            nRelated2 = JD_IMP_STEALTH;		
            nRelated3 = JD_MSTR_STEALTH;		
            break;
        case JD_IMP_STEALTH:					
            nRelated3 = JD_MSTR_STEALTH;		
            break;
        case JD_TREAT_INJURY:					
            nRelated2 = JD_IMP_TREAT_INJURY;	
            nRelated3 = JD_MSTR_TREAT_INJURY;	
            break;
        case JD_IMP_TREAT_INJURY:				
            nRelated3 = JD_MSTR_TREAT_INJURY;  
            break;
        case JD_RECK:    					
            nRelated2 = JD_RECK_IMP;		
            nRelated3 = JD_RECK_MSTR;		
            break;
        case JD_RECK_IMP:					
            nRelated3 = JD_RECK_MSTR;		
            break;
        case JD_RAGE:	       				
            nRelated2 = JD_RAGE_IMP;		
            nRelated3 = JD_RAGE_MSTR;		
            break;
        case JD_RAGE_IMP:					
            nRelated3 = JD_RAGE_MSTR;		
            break;
        case JD_HEAL_FORCE_MINOR:	       		
            nRelated2 = JD_HEAL_FORCE_AVERAGE;	
            nRelated3 = JD_HEAL_FORCE_STRONG;	
            break;
        case JD_HEAL_FORCE_AVERAGE:				
            nRelated3 = JD_HEAL_FORCE_STRONG;	
            break;
        default:
        	// ST: Power has no related ones, just check if for the specified power.
            return GetHasSpellEffect(nSpellID, oTarget);
            break;
    }

    if( 	GetHasSpellEffect(nSpellID, oTarget)
    	|| 	GetHasSpellEffect(nRelated1, oTarget)
        || 	GetHasSpellEffect(nRelated2, oTarget)
        || 	GetHasSpellEffect(nRelated3, oTarget)
        || 	GetHasSpellEffect(nRelated4, oTarget)
        || 	GetHasSpellEffect(nRelated5, oTarget))
    {
        return TRUE;
    }

    return FALSE;
}


// ----------------------------------------------------------------------
// ST_SubstPowerIfPresent()
// 		If the caller has the force power nPower2, return a new talent
//      for using it. If not, but the caller has nPower1 instead, return
//      a talent for using that one. If none of the specified powers are
//      available to the caller, return the original talent provided.
// ----------------------------------------------------------------------
talent ST_SubstPowerIfPresent(talent tUse, int nPower1, int nPower2 = -1) {
	talent tNew;
	if ((nPower2 != -1) && GetHasSpell(nPower2)) {
		tNew = TalentSpell(nPower2);
		if (GetIsTalentValid(tNew)) 
			return tNew;
	}	
	if (GetHasSpell(nPower1)) {
		tNew = TalentSpell(nPower1);
		if (GetIsTalentValid(tNew)) 
			return tNew;
	}	
	return tUse;
}


// ----------------------------------------------------------------------
// ST_SubstFeatIfPresent()
// 		If the caller has the force power nFeat2, return a new talent
//      for using it. If not, but the caller has nFeat1 instead, return
//      a talent for using that one. If none of the specified feats are
//      available to the caller, return the original talent provided.
// ----------------------------------------------------------------------
talent ST_SubstFeatIfPresent(talent tUse, int nFeat1, int nFeat2 = -1) {
	talent tNew;
	if ((nFeat2 != -1) && GetHasFeat(nFeat2)) {
		tNew = TalentFeat(nFeat2);
		if (GetIsTalentValid(tNew)) 
			return tNew;
	}	
	if (GetHasFeat(nFeat1)) {
		tNew = TalentFeat(nFeat1);
		if (GetIsTalentValid(tNew)) 
			return tNew;
	}	
	return tUse;
}


// ----------------------------------------------------------------------
// ST_GetCorrectForcePower()
// 		Pick the best force power in the same family as the one specified
//      by tUse, if one is present. If no better power was available,
//      the original talent will be returned.
// ----------------------------------------------------------------------
talent ST_GetCorrectForcePower(talent tUse) {
	if (GetTypeFromTalent(tUse) == TALENT_TYPE_FORCE) {
	    switch (GetIdFromTalent(tUse)) {
	    	
	    	// OFFENSIVE FORCE POWERS ----------------------------------------
	    	case FORCE_POWER_FEAR: 		
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_HORROR, FORCE_POWER_INSANITY);
	    		break;	   	    	
	    	case FORCE_POWER_HORROR: 		
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_INSANITY);
	    		break;
	    	case FORCE_POWER_DROID_STUN: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_DROID_DISABLE, FORCE_POWER_DROID_DESTROY); 
	    		break;  	    		
	    	case FORCE_POWER_DROID_DISABLE: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_DROID_DESTROY); 
	    		break;
	    	case FORCE_POWER_SHOCK: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_LIGHTNING, FORCE_POWER_FORCE_STORM); 
	    		break; 	    		
	    	case FORCE_POWER_LIGHTNING: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_FORCE_STORM); 
	    		break;    		
	    	case FORCE_POWER_FORCE_SCREAM:	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_IMPROVED_FORCE_SCREAM, FORCE_POWER_MASTER_FORCE_SCREAM); 
	    		break;
	    	case FORCE_POWER_IMPROVED_FORCE_SCREAM:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_MASTER_FORCE_SCREAM);
	    		break;    	
	    	case FORCE_POWER_WOUND: 		
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_CHOKE, FORCE_POWER_KILL);
	    		break;
	    	case FORCE_POWER_CHOKE: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_KILL); 
	    		break;
	    	case FORCE_POWER_SLOW:    		
	    	case FORCE_POWER_AFFLICTION: 
	    		// ST: Skip affliction on purpose, its Save DC is ridiculously insufficient.	    		
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_PLAGUE); 
	    		break;      	
	    	case FORCE_POWER_FORCE_PUSH: 	
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_FORCE_WHIRLWIND); 
	    		break;     	
	    	case FORCE_POWER_STUN:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_HOLD, FORCE_POWER_SLEEP);
	    		break;   	    		
	    	case FORCE_POWER_HOLD: 	// ST: Stasis
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_SLEEP); // ST: Stasis Field
	    		break;  
	    		
	    	// DEFENSIVE FORCE POWERS ------------------------------------------------------
	    	case FORCE_POWER_RESIST_POISON_DISEASE_SONIC: // ST: Resist Energy
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_RESIST_COLD_HEAT_ENERGY, FORCE_POWER_MASTER_ENERGY_RESISTANCE);
	    		break;  
	    	case FORCE_POWER_RESIST_COLD_HEAT_ENERGY:  // ST: Improved Resist Energy
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_MASTER_ENERGY_RESISTANCE);
	    		break;  	  
	    	case FORCE_POWER_FORCE_BARRIER:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_IMPROVED_FORCE_BARRIER, FORCE_POWER_MASTER_FORCE_BARRIER);
	    		break;  	    	
	    	case FORCE_POWER_IMPROVED_FORCE_BARRIER:  
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_MASTER_FORCE_BARRIER);
	    		break;  
	    	case FORCE_POWER_CURE:  // ST: Heal
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_HEAL, FORCE_POWER_MASTER_HEAL);
	    		break;  	
	    	case FORCE_POWER_HEAL:  // ST: Improved Heal
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_MASTER_HEAL);
	    		break; 
	    	case FORCE_POWER_SPEED_BURST:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_KNIGHT_SPEED, FORCE_POWER_SPEED_MASTERY);
	    		break;  	    	
	    	case FORCE_POWER_KNIGHT_SPEED:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_SPEED_MASTERY);
	    		break; 
	    	case FORCE_POWER_FORCE_AURA:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_FORCE_SHIELD, FORCE_POWER_FORCE_ARMOR);
	    		break;
	    	case FORCE_POWER_FORCE_SHIELD:
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_FORCE_ARMOR);
	    		break;
	    	case FORCE_POWER_FORCE_MIND: // ST: Valor
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_KNIGHT_MIND, FORCE_POWER_MIND_MASTERY);
	    		break;	   
	    	case FORCE_POWER_KNIGHT_MIND: // ST: Knight Valor
	    		tUse = ST_SubstPowerIfPresent(tUse, FORCE_POWER_MIND_MASTERY);
	    		break;	 	    		 		    		    		
	    }	    
	}
	return tUse;	
}


// ----------------------------------------------------------------------
// ST_GetCorrectCombatFeat()
// 		Pick the best combat feat in the same family as the one specified
//      by tUse, if one is present. If no better feat was available,
//      the original talent will be returned.
// ----------------------------------------------------------------------
talent ST_GetCorrectCombatFeat(talent tUse) {
	if (GetTypeFromTalent(tUse) == TALENT_TYPE_FEAT) {
	    switch (GetIdFromTalent(tUse)) {
	    	
	    	// ST: Melee feats ------------------------------------------
	    	case FEAT_CRITICAL_STRIKE: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_CRITICAL_STRIKE, FEAT_MASTER_CRITICAL_STRIKE);
	    		break;
	    	case FEAT_IMPROVED_CRITICAL_STRIKE: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_MASTER_CRITICAL_STRIKE);
	    		break;  		 
	    	case FEAT_FLURRY: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_FLURRY, FEAT_WHIRLWIND_ATTACK);
	    		break;   
	    	case FEAT_IMPROVED_FLURRY: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_WHIRLWIND_ATTACK);
	    		break;     		
	    	case FEAT_POWER_ATTACK: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_POWER_ATTACK, FEAT_MASTER_POWER_ATTACK);
	    		break;  
	    	case FEAT_IMPROVED_POWER_ATTACK: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_MASTER_POWER_ATTACK);
	    		break; 
	    		
	    	// ST: Ranged feats -----------------------------------------
	    	case FEAT_SNIPER_SHOT: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_SNIPER_SHOT, FEAT_MASTER_SNIPER_SHOT);
	    		break;   
	    	case FEAT_IMPROVED_SNIPER_SHOT: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_MASTER_SNIPER_SHOT);
	    		break; 
	    	case FEAT_RAPID_SHOT: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_RAPID_SHOT, FEAT_MULTI_SHOT);
	    		break;   
	    	case FEAT_IMPROVED_RAPID_SHOT: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_MULTI_SHOT);
	    		break;  
	    	case FEAT_POWER_BLAST: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_IMPROVED_POWER_BLAST, FEAT_MASTER_POWER_BLAST);
	    		break;    
	    	case FEAT_IMPROVED_POWER_BLAST: 		
	    		tUse = ST_SubstFeatIfPresent(tUse, FEAT_MASTER_POWER_BLAST);
	    		break;        		   		    		   		    		 	 		    		     		 		   		 
	    }	
	}
    return tUse;
}


// ----------------------------------------------------------------------
// ST_GetCorrectForceOrFeat()
// 		Wrapper function. If the specified talent is a force power, 
//      substitute it for the best available in the same family if one
//      is known by the caller. If it's a combat feat, do the same thing.
//      If the talent was neither a force power or a combat feat, or if
//      no better one was available, the original talent is returned.
// ----------------------------------------------------------------------
talent ST_GetCorrectForceOrFeat(talent tUse) {
	if (GetTypeFromTalent(tUse) == TALENT_TYPE_FORCE) {
		return ST_GetCorrectForcePower(tUse);
	}
	else if (GetTypeFromTalent(tUse) == TALENT_TYPE_FEAT) {
		return ST_GetCorrectCombatFeat(tUse);
	}
	return tUse;
}


// ----------------------------------------------------------------------
// ST_ShouldUseDisablingPower()
// 		Check if the target is already disabled by whirlwind, stun,
//      stasis or horror if the specified talent is one such ability, to
//      eliminate time wasted on trying to disable someone who is already
//      helpless. Returns TRUE if the target is not disabled by such a
//      power and as such the talent is OK to use, otherwise FALSE.
//
// FIX 2005-07-30: Added Whirlwind to powers to check for.
//
// FIX 2005-07-30: Added additional power filters, despite the function name...
// ----------------------------------------------------------------------
int ST_ShouldUseDisablingPower(talent tUse, object oTarget) {
	if (GetTypeFromTalent(tUse) == TALENT_TYPE_FORCE) {
		int nId = GetIdFromTalent(tUse);
		
		if ((GetIsDebilitated(oTarget)
			    || ST_GetIsHelpless(oTarget))
			&& (	(nId == FORCE_POWER_SLEEP)
				|| 	(nId == FORCE_POWER_HOLD)
				|| 	(nId == FORCE_POWER_STUN)
				||  (nId == FORCE_POWER_FORCE_WHIRLWIND)
				|| 	(nId == FORCE_POWER_HORROR)
				|| 	(nId == FORCE_POWER_INSANITY)
				|| 	(nId == FORCE_POWER_FEAR)
				||  (nId == FORCE_POWER_MIND_TRICK)
				||  (nId == FORCE_POWER_BEAST_TRICK)
				||  (nId == FORCE_POWER_DROID_TRICK)
				||  (nId == FORCE_POWER_DROID_STUN))
			&& (	GetHasSpellEffect(FORCE_POWER_SLEEP, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_HOLD, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_STUN, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_FORCE_WHIRLWIND, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_HORROR, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_INSANITY, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_FEAR, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_MIND_TRICK, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_BEAST_TRICK, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_DROID_TRICK, oTarget)
				|| 	GetHasSpellEffect(FORCE_POWER_DROID_STUN, oTarget)))
		{
			return FALSE;
		}
		
		// ST: Don't use poisoning powers if the opponent is immune (droid)
		if (((nId == FORCE_POWER_PLAGUE) || (nId == FORCE_POWER_AFFLICTION))
			&& (GetIsPoisoned(oTarget)  
				|| GetIsImmune(oTarget, IMMUNITY_TYPE_POISON) 
				|| (GetRacialType(oTarget) == RACIAL_TYPE_DROID)))
		{
			return FALSE;	
		}
		
		// ST: Don't use stun power if opponent is immune	
		if ((nId == FORCE_POWER_STUN) 
			&& (GetIsImmune(oTarget, IMMUNITY_TYPE_STUN) || GetFeatAcquired(99, oTarget))) 
		{
			return FALSE;	
		}
		
		// ST: Don't use horror power if opponent is immune (ie droids)
		if (((nId == FORCE_POWER_FEAR) 
				|| (nId == FORCE_POWER_HORROR) 
				|| (nId == FORCE_POWER_INSANITY))
			&& (GetIsImmune(oTarget, IMMUNITY_TYPE_FEAR) 
				|| GetFeatAcquired(98, oTarget)
				|| GetFeatAcquired(200, oTarget)
				|| (GetRacialType(oTarget) == RACIAL_TYPE_DROID)))
		{
			return FALSE;	
		}
		
		// ST: Don't use Stasis if target is immune to paralysis
		//     (Don't check for Stasis Field since it does more than just paralyze,
		//      and might hit other nearby foes who are not immune as well.)
		if ((nId == FORCE_POWER_HOLD) 
			&& (GetIsImmune(oTarget, IMMUNITY_TYPE_PARALYSIS) || GetFeatAcquired(100, oTarget)))  
		{
			return FALSE;
		}
		
		// ST: Don't use Destroy Droid if the target is not a droid! 
		//     Seems to happen occasionally for some reason...
		if (((nId == FORCE_POWER_DROID_DESTROY) || (nId == FORCE_POWER_DROID_DISABLE) || (nId == FORCE_POWER_DROID_STUN))
			&& (GetRacialType(oTarget) != RACIAL_TYPE_DROID))
		{
			return FALSE;	
		}

	}
	
	return TRUE;
}


// ----------------------------------------------------------------------
// ST_GetAttackRange()
//		Returns the Max attack range of the weapon oAttacker currently
//		has equipped.
//
// 		EDIT: Changed to use preferred attack distance instead to eliminate
//            needless complexity and avoid having to check for off-hand
//            weapons to see which has the shortest range. The AI usually
//            only shoot from the prefattackdist or closer anyway...
// ----------------------------------------------------------------------
float ST_GetAttackRange(object oAttacker = OBJECT_SELF) {
	object oWeapon = GetItemInSlot(INVENTORY_SLOT_RIGHTWEAPON, oAttacker);
	float fRange;
	
	// ST: Unarmed attack!
	if (!GetIsObjectValid(oWeapon)) {
		fRange = 0.5;
	}
	else {
		// ST: Use PreferedRange instead of maxrange for simplicity...
		switch (GetBaseItemType(oWeapon)) {
			case BASE_ITEM_QUARTER_STAFF:
			case BASE_ITEM_STUN_BATON:
			case BASE_ITEM_LONG_SWORD:
			case BASE_ITEM_VIBRO_SWORD:
			case BASE_ITEM_SHORT_SWORD:
			case BASE_ITEM_VIBRO_BLADE:
			case BASE_ITEM_DOUBLE_BLADED_SWORD:
			case BASE_ITEM_VIBRO_DOUBLE_BLADE:
			case BASE_ITEM_LIGHTSABER:
			case BASE_ITEM_DOUBLE_BLADED_LIGHTSABER:
			case BASE_ITEM_SHORT_LIGHTSABER:
			case BASE_ITEM_GHAFFI_STICK:
			case BASE_ITEM_WOOKIE_WARBLADE:
			case BASE_ITEM_GAMMOREAN_BATTLEAXE:
			case BASE_ITEM_FORCE_PIKE:					
				fRange = 0.5; 
				break;
				
			case BASE_ITEM_BLASTER_PISTOL: 
			case BASE_ITEM_HEAVY_BLASTER:	
			case BASE_ITEM_HOLD_OUT_BLASTER:
			case BASE_ITEM_ION_BLASTER:
			case BASE_ITEM_DISRUPTER_PISTOL:
			case BASE_ITEM_SONIC_PISTOL:
			case BASE_ITEM_ION_RIFLE:
			case BASE_ITEM_BOWCASTER:
			case BASE_ITEM_BLASTER_CARBINE:
			case BASE_ITEM_DISRUPTER_RIFLE:
			case BASE_ITEM_SONIC_RIFLE:
			case BASE_ITEM_REPEATING_BLASTER:
			case BASE_ITEM_HEAVY_REPEATING_BLASTER:
			case BASE_ITEM_BLASTER_RIFLE:
				fRange = 20.0;
				break;
		}	
	}
	return fRange;
}


// ----------------------------------------------------------------------
// ST_GetNextShieldId()
// 		Returns the SpellId for the next Shield in sequence from the
//      number specified as a parameter.
// ----------------------------------------------------------------------
int ST_GetNextShieldId(int nId) {
	if (nId < 99)
		nId = 99;
	else if ((nId >= 99) && (nId < 107))
		nId++;
	else if (nId == 107)
		nId = 110;
	else if ((nId >= 110) && (nId < 115))
		nId++;
	else if (nId == 115)
		nId = 132;
	else if (nId == 132)
		nId = 257;
	else if (nId == 257)
		nId = 999;	
	return nId;
}


// ----------------------------------------------------------------------
// ST_GetIsShielded()
// 		Returns TRUE if the specified target is protected by an energy
// 		shield of some kind.
// ----------------------------------------------------------------------
int ST_GetIsShielded(object oTarget=OBJECT_SELF) {
	int nId;	
	for (nId = 99; nId != 999; nId = ST_GetNextShieldId(nId)) {
		if (GetHasSpellEffect(nId, oTarget))
			return TRUE;		
	} 	
	return FALSE;
}


// ----------------------------------------------------------------------
// ST_GetTrapInPathTo()
// 		Returns TRUE if there is a trap in the straight path between
// 		the caller and the specified target, if the current active party
// 		member does not have the Point Guard feat.
// ----------------------------------------------------------------------
int ST_GetTrapInPathTo(object oTarget) {
	if (GetIsObjectValid(oTarget) 
		&& !GetHasFeat(239, GetPartyMemberByIndex(0))
		&& (GetDistanceToObject2D(oTarget) > 1.0)) 
	{
		float fDist = GetDistanceToObject2D(oTarget);
		location lLoc = GetLocation(oTarget);
		object oTrap = 	GetFirstObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_TRIGGER | OBJECT_TYPE_PLACEABLE);
		while (GetIsObjectValid(oTrap)) {
			if (GetIsTrapped(oTrap) 
				&& GetTrapDetectable(oTrap) 
				&& (GetTrapDetectedBy(oTrap, OBJECT_SELF) || GetTrapDetectedBy(oTrap, GetPartyLeader()))
				&& !GetIsFriend(GetTrapCreator(oTrap))) 
			{
				// DebugMessage("ST_GetTrapInPathTo - Found a trap in the checked path!");
				return TRUE;
			}
			oTrap = GetNextObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_TRIGGER | OBJECT_TYPE_PLACEABLE);
		}
	}
	return FALSE;
}


// ----------------------------------------------------------------------
// ST_GetDoorInPathTo()
// 	Returns TRUE if there is a closed door in the straight path between
// 	the caller and the specified target.
// ----------------------------------------------------------------------
int ST_GetDoorInPathTo(object oTarget) {
	float fDist = GetDistanceToObject2D(oTarget);
	if (GetIsObjectValid(oTarget) && (fDist > 1.0)) {
		location lLoc = GetLocation(oTarget);
		object oDoor = 	GetFirstObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_DOOR);
		while (GetIsObjectValid(oDoor)) {
			if (!GetIsOpen(oDoor))  {
				// DebugMessage("ST_GetDoorInPathTo - Found a closed door in the checked path!");
				return TRUE;
			}
			oDoor = GetNextObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_DOOR);
		}
	}
	return FALSE;
}


// ----------------------------------------------------------------------
// Returns TRUE if there aren't any other creatures or placeables in a
// straight line between the caller and oTarget.
// ----------------------------------------------------------------------
int ST_GetHasClearPathTo(object oTarget) {
	if (GetIsObjectValid(oTarget) && GetObjectSeen(oTarget)) {
		float fDist = GetDistanceToObject2D(oTarget);
		location lLoc = GetLocation(oTarget);
		object oBlocker = GetFirstObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_CREATURE | OBJECT_TYPE_PLACEABLE);	
		while (GetIsObjectValid(oBlocker)) {
			if ((oBlocker != oTarget) && (oBlocker != OBJECT_SELF))
				return FALSE;
				
			oBlocker = GetNextObjectInShape(SHAPE_SPELLCYLINDER, fDist, lLoc, TRUE, OBJECT_TYPE_CREATURE | OBJECT_TYPE_PLACEABLE);		
		}
		return TRUE;
	}
	return FALSE;
}


// ----------------------------------------------------------------------
// Check if oTarget is a valid target for oAttacker, taking into account
// reputation, line of sight, visibility and distance between the two.
// Use this for PARTY MEMBERS ONLY, and only when they should stay close
// to the party leader.
//
// FIX: GetObjectSeen() does not work on the player controlled character,
//      it reports visibility for ALL creatures on the level no matter
//      where they are. Remove check, and try to replace that check with
//      a match against who the leader is fighting instead to avoid
//      standing around like an idiot around a corner while the leader
//      is visibly fighting someone yet unseen.
//
// FIX: Don't check if there is a trap in the way if using ranged
//      weapons, since you can shoot over the mines without having to
//      trigger them to attack the enemy.
//
// WTF: HasLineOfSight() is extremely unreliable, it sometimes returns
//      false for creatures standing right in front of the source, and
//      sometimes returns TRUE for creatures behind walls and doors.
//      Kill it, it causes more trouble than it solves, currently... :/
// ----------------------------------------------------------------------
int ST_IsValidAttackTarget(object oTarget, object oLeader, float fDist) {
	object oAttacker = OBJECT_SELF;
	
	if (GetIsObjectValid(oTarget)
		&& GetIsEnemy(oTarget, oAttacker)
		&& !GetIsDead(oTarget)
		&& (GetDistanceBetween2D(oLeader, oTarget) <= fDist)
		&& !ST_GetDoorInPathTo(oTarget)
		// && (HasLineOfSight(GetPosition(oAttacker), GetPosition(oTarget), oAttacker, oTarget) || HasLineOfSight(GetPosition(oLeader), GetPosition(oTarget), oLeader, oTarget))
    	&& (GetObjectSeen(oTarget, oAttacker) || (GetLastAttacker(oLeader) == oTarget) || (GetAttackTarget(oLeader) == oTarget))
    	&& (!ST_GetTrapInPathTo(oTarget) || ST_UsesRangedWeapon()))
    {
    	return TRUE;
    }
    
    /*
    DebugMessage("ST_IsValidAttackTarget(" + GetTag(oTarget) + ") returned FALSE!"
    	+ " Enemy=" + IntToString(GetIsEnemy(oTarget, oAttacker))
    	+ ", Dead=" + IntToString(GetIsDead(oTarget))
    	+ ", Distance=" + FloatToString(GetDistanceBetween2D(oLeader, oTarget), 4, 2)
    	+ ", DoorInPath=" + IntToString(ST_GetDoorInPathTo(oTarget))
    //	+ ", LoS(self)=" + IntToString(HasLineOfSight(GetPosition(oAttacker), GetPosition(oTarget), oAttacker, oTarget))
    //	+ ", LoS(lead)=" + IntToString(HasLineOfSight(GetPosition(oLeader), GetPosition(oTarget), oLeader, oTarget))
    	+ ", Seen(self)=" + IntToString(GetObjectSeen(oTarget, oAttacker))
    	+ ", Attacker(lead)=" + IntToString((GetLastAttacker(oLeader) == oTarget))
    	+ ", Attackee(lead)=" + IntToString((GetAttackTarget(oLeader) == oTarget))
    	+ ", TrapInPath=" + IntToString((ST_GetTrapInPathTo(oTarget) && !ST_UsesRangedWeapon())));
    */
    return FALSE;
}


// ----------------------------------------------------------------------
// Retrieves a target within the allowed radius that can be perceived.
// Checks distance from party leader, if target can be seen and no
// doors are in the way and tries to prevent stepping on mines.
// ----------------------------------------------------------------------
object ST_GetTargetInRange(float fDist, object oTarget, object oIntruder=OBJECT_INVALID) {
	object oLeader = GetPartyLeader();
	object oClose;
	int nIdx = 1;
	
	// FIX(2005-08-31) Redo a prioritized target selection once more, but with
	//                 the current attack target cleared and see if we get someone
	//                 else who's within range this time.
	if (!ST_IsValidAttackTarget(oTarget, oLeader, fDist)) {	
		ST_SetLastAttackTarget(OBJECT_INVALID);
		oTarget = ST_SelectAttackTarget();
	}
	
	if (!ST_IsValidAttackTarget(oTarget, oLeader, fDist)) {		
		if (ST_IsValidAttackTarget(oIntruder, oLeader, fDist)) {
			oTarget = oIntruder;
		}
		else {
    		oIntruder = OBJECT_INVALID;
    		oTarget = OBJECT_INVALID;
    		
    		oClose = GetLastHostileTarget();
    		if (ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
    			oTarget = oClose;
    		}
    		else {
    			oClose = GetLastAttacker(OBJECT_SELF);
    			if (ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
    				oTarget = oClose;
    			}
    			else {
	    			oClose = GetLastAttacker(oLeader);
	    			if (ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
	    				oTarget = oClose;
	    			}
	    			else {
		    			oClose = GetAttackTarget(oLeader);
		    			if (ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
		    				oTarget = oClose;
		    			}
		    			else {
		    				oClose = GetLastPerceived();
		    				if (GetLastPerceptionSeen() && ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
		    					oTarget = oClose;
		    				}
			    			else {
			    				// ST: If this continues to goof up, try to use ObjectInShape to determine a
			    				//     new enemy instead, and see if it's more reliable? Doubtful it would make
			    				//     any difference since it has to go through ST_IsValidAttackTarget either way.
					    		oClose = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, oLeader, nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
					    		
					    		while (GetIsObjectValid(oClose)) {
					    			if (ST_IsValidAttackTarget(oClose, oLeader, fDist)) {
					    				// DebugMessage("ST_GetTargetInRange - OCLOSE (" + GetName(oClose) + ") is a valid target, RETARGETING! dist="  + FloatToString(GetDistanceBetween2D(oLeader, oClose), 4, 1));
					    				oTarget = oClose;
					    				break;
					    			}
					    			oClose = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, oLeader, ++nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
					    		}	
					    		
					    		//if (oTarget == OBJECT_INVALID)
					    		//	DebugMessage("ST_GetTargetInRange - No valid target found after all checks have been done!");    				
					    		
			    			}
		    			}	    				
	    			}    				
    			}
    		}
		}		
	}	
	return oTarget;
}


// ----------------------------------------------------------------------
// Returns TRUE if oTarget is either a Party Member or a Party Puppet.
// ----------------------------------------------------------------------
int ST_IsPartyMemberOrAssociate(object oTarget = OBJECT_SELF) {
	return (IsObjectPartyMember(oTarget) || (GetStandardFaction(oTarget) == STANDARD_FACTION_PARTYPUPPET));
}


// ----------------------------------------------------------------------
// Returns TRUE if oTarget is affected by a Rage feat.
// FIX(2005-08-31) Added checks for the spell effects too...
// ----------------------------------------------------------------------
int ST_GetIsEnraged(object oTarget) {
    return (GetHasFeatEffect(233, oTarget) 
    	|| GetHasFeatEffect(232, oTarget) 
    	|| GetHasFeatEffect(231, oTarget)
    	|| GetHasSpellEffect(166, oTarget)
    	|| GetHasSpellEffect(165, oTarget)
    	|| GetHasSpellEffect(164, oTarget)
    	|| GetHasSpellEffect(FORCE_POWER_WOOKIEE_RAGE_III, oTarget)
    	|| GetHasSpellEffect(FORCE_POWER_WOOKIEE_RAGE_II, oTarget)
    	|| GetHasSpellEffect(FORCE_POWER_WOOKIEE_RAGE_I, oTarget));
}


// ----------------------------------------------------------------------
// Checks if oTarget knows a particular High Level FP. If it's a buff,
// check that it isn't already active on the force user. Also only
// return true about 50% of the time no matter what, to provide some
// random variety.
// ----------------------------------------------------------------------
int ST_GetHasHLSpell(int nSpell, object oTarget=OBJECT_SELF, int bBuff=FALSE) {
	return ((ST_GLOBAL_ENABLE_POWERS == TRUE) 
		&& GetHasSpell(nSpell, oTarget)
		&& (!bBuff || (bBuff && !GetHasSpellEffect(nSpell, oTarget))));
}


// ---------------------------------------------------------------
// UTILITY: Return string value of nVal zero-padded to the number
//          of digits specified by the nDigits parameter.
// ---------------------------------------------------------------
string IntToStringPad(int nVal, int nDigits) {
	string sRes = IntToString(nVal);
	if (sRes != "") {
		int nDiff = nDigits - GetStringLength(sRes);
		
		if (nDiff > 0) {
			int i;
			for (i = 0; i < nDiff; i++) {
				sRes = "0" + sRes;	
			}	
		}
	}
	return sRes;
}


// --------------------------------------------------------------------
// ST: TIME FOR UGLY HACKS...
//     Since GetLastHostileTarget() doesn't seem to return the last attacked
//     target all too often, try to hijack the only function that allows storing an
//     object reference for a creature for this purpose. Store the target
//     whenever an attack is made or a talent is used with this function!!
// ---------------------------------------------------------------
void ST_SetLastAttackTarget(object oTarget) {
	if (GetNPCAIStyle(OBJECT_SELF) == NPC_AISTYLE_HEALER)
		return;
	
	if (GetIsObjectValid(oTarget) && GetIsEnemy(oTarget) && !GetIsDead(oTarget)) {
		SetHealTarget(OBJECT_SELF, oTarget);		
	}
	else {
		SetHealTarget(OBJECT_SELF, OBJECT_INVALID);		
	}
}


// ---------------------------------------------------------------
// ST: TIME FOR UGLY HACKS...
// Retrieve the stored last attack target, if it's a valid enemy.
// Try to counter player tactic of making them chase the player
// around while party members kill them by dropping the player
// as a target if they are moving. Use this instead of the
// GetLastHostileTarget() function to get the last target.
// ---------------------------------------------------------------
object ST_GetLastAttackTarget() {
	if (GetNPCAIStyle(OBJECT_SELF) == NPC_AISTYLE_HEALER)
		return GetLastHostileTarget();
	
	object oTarg = GetHealTarget(OBJECT_SELF);
	
	if (GetIsObjectValid(oTarg) 
		&& GetIsEnemy(oTarg) 
		&& !GetIsDead(oTarg) 
		&& GetObjectSeen(oTarg)
		&& (GetCurrentHitPoints(oTarg) > 0)) 
	{
		// If it's the player character, which isn't attacking me and is moving,
		// drop it and try someone else. Might be the player leading me in circles while
		// the party members attack with impunity.
		if (oTarg == GetPartyLeader() 
			&& (GetAttackTarget(oTarg) != OBJECT_SELF)
			&& (GetCurrentAction(oTarg) == ACTION_MOVETOPOINT)) 
		{
			SetHealTarget(OBJECT_SELF, OBJECT_INVALID);
			return OBJECT_INVALID;	
		}
		
		// Return the last attack target
		return oTarg;
	}
	else if (GetIsObjectValid(oTarg)) {
		SetHealTarget(OBJECT_SELF, OBJECT_INVALID);	
	}
		
	return OBJECT_INVALID;
}

// ---------------------------------------------------------------
// Returns TRUE if oTarget has any level in Jedi/Sith classes
// ---------------------------------------------------------------
int ST_GetIsForceUser(object oTarget=OBJECT_SELF) {
	return ((GetLevelByClass(3, oTarget) 
		+ GetLevelByClass(4, oTarget) 
		+ GetLevelByClass(5, oTarget) 
		+ GetLevelByClass(11, oTarget)
		+ GetLevelByClass(12, oTarget)
		+ GetLevelByClass(13, oTarget)
		+ GetLevelByClass(14, oTarget)
		+ GetLevelByClass(15, oTarget)
		+ GetLevelByClass(16, oTarget)) > 0);	
}


int ST_GetPerformsCombatAction(object oTarget) {
	int nAction = GetCurrentAction(oTarget);
	return ((nAction == ACTION_ATTACKOBJECT) 
		|| (nAction == ACTION_CASTSPELL)  
		|| (nAction == 39)
		|| (nAction == ACTION_SETTRAP)
		|| (nAction == ACTION_ITEMCASTSPELL)
		|| (nAction == ACTION_HEAL));
}


// ---------------------------------------------------------------
// Support function for ST_SelectAttackTarget(). Calculate a
// priority value for a creature depending on a number of factors.
// oTarget = Creature to check
// oLast   = Creature I attacked most recently
// oVersus = The creature doing the attacks, usually OBJECT_SELF.
// ---------------------------------------------------------------
int ST_CalculatePriority(object oTarget, object oLast, object oVersus=OBJECT_SELF) {
	int nPriority = 0;
	int bSelfRanged = ST_UsesRangedWeapon(oVersus);
	int bTargRanged = ST_UsesRangedWeapon(oTarget);
	
	// See if the attacker has Sneak Attack. Better focus on those first
	// since they deal extra damage if I don't focus on them...		
	// FIX(2005-08-31) Ignore sneak attack criteria if I'm immune to it.
	if (!GetIsImmune(oVersus, IMMUNITY_TYPE_SNEAK_ATTACK)) {
		if (GetHasFeat(FEAT_SNEAK_ATTACK_10D6, oTarget))
			nPriority += 5;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_9D6, oTarget))
			nPriority += 5;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_8D6, oTarget))
			nPriority += 4;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_7D6, oTarget))
			nPriority += 3;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_6D6, oTarget))
			nPriority += 3;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_5D6, oTarget))
			nPriority += 2;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_4D6, oTarget))
			nPriority += 2;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_3D6, oTarget))
			nPriority += 1;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_2D6, oTarget))
			nPriority += 1;
		else if (GetHasFeat(FEAT_SNEAK_ATTACK_1D6, oTarget))
			nPriority += 1;	
	}	
		
	// Give slightly more focus to less powerful opponents first
	// to eliminate any weaker supporting characters more quickly.
	if (GetHitDice(oVersus) > GetHitDice(oTarget))
		nPriority += 1;
		
	// If using a melee weapon, go after nearby opponents first to reduce
	// needless running around.
	if (!bSelfRanged && (GetDistanceBetween2D(oTarget, oVersus) < 4.0)) 
		nPriority += 4;
		
	// If they are using a ranged weapon, I use a melee weapon and they
	// are close, take them out since they get an insane pointblank-bonus to hit but also defense penalty.
	if (!bSelfRanged && bTargRanged && (GetDistanceBetween2D(oTarget, oVersus) < 4.0))
		nPriority += 4;
	
	// Give priority to enemies depending on how injured they are. Take out enemies
	// who are near death first, since they can harm you just as good as an uninjured one if left alone.
	nPriority += FloatToInt((1.0 - IntToFloat(GetCurrentHitPoints(oTarget)) / IntToFloat(GetMaxHitPoints(oTarget))) * 10.0);
	
	// If this is the last target we attacked, give good priority to them
	// to avoid switching around targets all the time. Focus on one, unless
	// the situation really warrants a change.
	if (oTarget == oLast)
		nPriority += 8;
	
	// Give priority bonus to someone who's attacking me...
	if (GetLastAttacker(oTarget) == oVersus)
		nPriority += 4;
		
	// Give priority bonus to someone who I'm already attacking...
	if (GetLastAttacker(oVersus) == oTarget)
		nPriority += 3;		
			
	// If this target isn't fighting currently, don't provoke them needlessly if
	// there are other opponents nearby who are already fighting.
	if (!GetIsInCombat(oTarget, TRUE) && !GetIsDebilitated(oTarget))
		nPriority -= 4;
		
	// Don't chase people around needlessly if someone you can deal damage
	// to is available instead.
	if (GetCurrentAction(oTarget) == ACTION_MOVETOPOINT)
		nPriority -= 5;
	
	// Apply penalty if the target isn't involved in any combat actions...
	// Take out those who present an immediate danger first.
	if (!ST_GetPerformsCombatAction(oTarget))
		nPriority -= 4;
		
	// Give some priority to attacking opponents with a Jedi class since
	// their force powers can be dangerous if left alone too long.
	if (ST_GetIsForceUser(oTarget))
		nPriority += 2;
	
	
	// Add priority penalty if using blaster weapons against an
	// opponent with a lightsaber. How much depends on if I know
	// any of the Precise Shot feats that counter BBD.			
	object oWeapon = GetItemInSlot(INVENTORY_SLOT_RIGHTWEAPON, oTarget);
	if (GetIsObjectValid(oWeapon)) {
		int nBase = GetBaseItemType(oWeapon);
		switch (nBase) {
			case BASE_ITEM_LIGHTSABER:	
			case BASE_ITEM_DOUBLE_BLADED_LIGHTSABER: 
			case BASE_ITEM_SHORT_LIGHTSABER:
				if (bSelfRanged) {
					if (GetHasFeat(FEAT_PRECISE_SHOT_V, oVersus))
						nPriority += 0;
					else if (GetHasFeat(FEAT_PRECISE_SHOT_IV, oVersus))
						nPriority += 0;
					else if (GetHasFeat(FEAT_MASTER_PRECISE_SHOT, oVersus))
						nPriority -= 1;
					else if (GetHasFeat(FEAT_IMPROVED_PRECISE_SHOT, oVersus))
						nPriority -= 2;
					else if (GetHasFeat(FEAT_PRECISE_SHOT, oVersus))
						nPriority -= 3;	
					else
						nPriority -= 4;				
				}
				// Saber is more dangerous than most melee weapons. Focus on them...
				else {
					nPriority += 2;	
				}
				break;
		}
		
		// ADDED(2005-09-09)
		// If the target is dual-wielding, give extra priority since they can deal out
		// more damage than other opponents. At least in theory. :)
		if ((nBase == BASE_ITEM_DOUBLE_BLADED_LIGHTSABER)
			|| (nBase == BASE_ITEM_QUARTER_STAFF)
			|| (nBase == BASE_ITEM_DOUBLE_BLADED_SWORD)
			|| (nBase == BASE_ITEM_VIBRO_DOUBLE_BLADE)
			|| (nBase == BASE_ITEM_FORCE_PIKE)
			|| GetIsObjectValid(GetItemInSlot(INVENTORY_SLOT_LEFTWEAPON, oTarget)))
		{
			nPriority += 2;
		}
	}
	
	// De-prioritize heavily if target is stealthed, since they would be
	// hard to focus on in the heat of battle even if I know they are there,
	// and they can't attack while in stealth anyway.
	if (IsStealthed(oTarget))
		nPriority -= 8;
	
	// Since the player char is usually more powerful than the rest of
	// the party members, add this to prevent them from all ganging up on
	// the Exile all the time.
	if (!GetIsPlayerMadeCharacter(oTarget) && IsObjectPartyMember(oTarget)) 
		nPriority += 2;		
	

	// ADDED(2005-09-09)
	// If I'm a melee fighter, give some priority to ranged attackers with the higher 
	// Precise Shot feats and good Dex since they do extra damage and can more easily 
	// bypass the Blaster Bolt Def.
	if (bTargRanged && !bSelfRanged && (GetAbilityScore(oTarget, ABILITY_DEXTERITY) >= 16)) {
		if (GetHasFeat(FEAT_PRECISE_SHOT_V, oTarget))
			nPriority += 3;
		else if (GetHasFeat(FEAT_PRECISE_SHOT_IV, oTarget))
			nPriority += 2;
		else if (GetHasFeat(FEAT_MASTER_PRECISE_SHOT, oTarget))
			nPriority += 1;
	}
	
	if (!bTargRanged) {
		// If the strength modifier of this creature is greater than mine, and they
		// are using melee weapons and have any significant strength, they are dangerous 
		// to leave around for too long. Try to eliminate them early.
		// FIX(2005-09-09): Added lower strength range limit to make it more meaningful...
		int nTargStr = GetAbilityModifier(ABILITY_STRENGTH, oTarget);
		int nSelfStr = GetAbilityModifier(ABILITY_STRENGTH, oVersus);
		if ((nTargStr > nSelfStr) && (nTargStr > 2))
			nPriority += ST_IMin(nTargStr - nSelfStr, 4);		
	}
	
	return nPriority;
}


// ---------------------------------------------------------------
// Get the total current hitpoints of all valid, living enemies
// that the caller can perceive.
// ---------------------------------------------------------------
int ST_GetEnemyHPTotal() {
	int nHP = 0;
	int nIdx = 1;
	
	object oTarget = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, OBJECT_SELF, nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
	while (GetIsObjectValid(oTarget)) {	
		if 	(GetIsEnemy(oTarget) 
			&& (GetDistanceBetween2D(oTarget, OBJECT_SELF) < 30.0)
			&& GetObjectSeen(oTarget)
			&& !GetIsDead(oTarget) 
			&& (GetCurrentHitPoints(oTarget) > 0)) 
		{
			nHP += GetCurrentHitPoints(oTarget);
		}
		oTarget = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, OBJECT_SELF, ++nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
	}	
	return nHP;
} 


// ---------------------------------------------------------------
// Attempt to pick a target to attack depending on the situation,
// rather than just pick the closest opponent.
// This should be used by the BOSS AI to be a bit smarter in 
// picking who to attack.
// ---------------------------------------------------------------
object ST_SelectAttackTarget(object oIntruder=OBJECT_INVALID) {
	// If oIntruder is specified, try to use that one first...
	if (GetIsObjectValid(oIntruder) 
		&& GetIsEnemy(oIntruder) 
		&& !GetIsDead(oIntruder)
		&& (GetCurrentHitPoints(oIntruder) > 0))	
	{
		return oIntruder;	
	}
		
	int nLastPriority = -1000;
	int nPriority     = 0;
	object oSelected  = OBJECT_INVALID;
	object oLast      = ST_GetLastAttackTarget();
	object oTarget;
		
	// Pick an opponent among nearby enemies based on a number of criteria...
	if (!GetIsObjectValid(oSelected)) {
		// ADDED(2005-09-09)
		// TEST! See if it slows down things too much to loop through enemies twice!
		// If there's a noticable change, remove this again...
		int nHPTotal = ST_GetEnemyHPTotal();		
		int nIdx = 1;
		
		oTarget = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, OBJECT_SELF, nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
		while (GetIsObjectValid(oTarget)) {
			if 	(GetIsEnemy(oTarget) 
				&& (GetDistanceBetween2D(oTarget, OBJECT_SELF) < 30.0)
				&& GetObjectSeen(oTarget)
				&& !GetIsDead(oTarget) 
				&& (GetCurrentHitPoints(oTarget) > 0)) 
			{
				// Get the priority of this creature...
				nPriority = ST_CalculatePriority(oTarget, oLast);
				
				// Add slight randomizer for variation/whim...
				if (d2() == 1) {
					nPriority += Random(3);	
				}
				
				// ADDED(2005-09-09) - Extra filter (TEST IF THIS IS A GOOD IDEA):
				// Give somewhat higher priority to enemies with fewer hitpoints, since they die quicker
				// and it should be easier to get rid of the risk they pose first.
				if (nHPTotal > 0)
					nPriority -= ST_IMin(FloatToInt((IntToFloat(GetCurrentHitPoints(oTarget))/IntToFloat(nHPTotal)) * 10.0), 4);
				
				// If the priority of this creature is larger than any of the previously
				// checked ones, pick this one as target for now...
				if (nPriority > nLastPriority) {
					nLastPriority = nPriority;
					oSelected     = oTarget;
				}
			}			
			oTarget = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, OBJECT_SELF, ++nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);
		}		
	}
	
	// Return the selected target, if any.
	return oSelected;
}


// ---------------------------------------------------------------
// Wrapper support function used in AssignHLPowers():
// ---------------------------------------------------------------
void ST_GiveForcePower(int nPower) {
	if (!GetSpellAcquired(nPower))
		GrantSpell(nPower, OBJECT_SELF);
}


// ---------------------------------------------------------------
// Assign High level Force Powers to self if I am one of the
// big bosses in the game, and it is enabled.
// ---------------------------------------------------------------
void ST_AssignHLPowers(int bBlock=FALSE) {
	// ST: Added THIS! If HLFP are enabled, assign them to some of the BOSSES!
	if (!IsObjectPartyMember(OBJECT_SELF)
		&& (ST_GLOBAL_ENABLE_POWERS == TRUE)
		&& (ST_GLOBAL_ENABLE_BOSS_POWERS == TRUE)
		&& !GetLocalBoolean(OBJECT_SELF, 88) // SW_CANDEROUS_COMBAT_REGEN		
	//	&&  GetLocalBoolean(OBJECT_SELF, 89) // SW_FLAG_BOSS_AI
		&& ST_GetIsForceUser(OBJECT_SELF)) 
	{
		// ST: This will now be set when assigning saber forms instead to save a Var...
		if (bBlock)
			SetLocalBoolean(OBJECT_SELF, 88, TRUE);
		
		switch (GetAppearanceType(OBJECT_SELF)) {
			case 458: // Darth Sion
				ST_GiveForcePower(JD_LEECH_MSTR);
				ST_GiveForcePower(JD_CONTAGION_MSTR);
				break;
				
			case 463: // Darth Nihilus
				ST_GiveForcePower(JD_WEAKEN_MSTR);
				ST_GiveForcePower(JD_LEECH_MSTR);
				break;
							
			case 600:
			case 651: // Darth Traya
				ST_GiveForcePower(JD_INFERNO_MSTR);
				ST_GiveForcePower(JD_CHAIN_MSTR);
				break;			
			
			case 649: // Master Vash
				ST_GiveForcePower(JD_REGEN_MSTR);
				ST_GiveForcePower(JD_OMNI_MSTR);
				break;				
			
			case 670: // Master Vrock
				ST_GiveForcePower(JD_SAPWILL_MSTR);
				ST_GiveForcePower(JD_PROTECT_MSTR);
				break;				
			
			case 648: // Master O'Kavar
				ST_GiveForcePower(JD_ENERGY_MSTR);
				ST_GiveForcePower(JD_HURRICANE_MSTR);
				break;	
							
			case 641: // Master Says Kai Ell
				ST_GiveForcePower(JD_ENERGY_MSTR);
				ST_GiveForcePower(JD_REGEN_MSTR);
				break;	
							
			case 629: // "Master" Atris - Frost Queen Special :)
				ST_GiveForcePower(JD_FREEZE_MSTR);
				break;				
			
			case  52: // Master Vandar	
				ST_GiveForcePower(JD_REGEN_MSTR);
				ST_GiveForcePower(JD_ENERGY_MSTR);
				ST_GiveForcePower(JD_HURRICANE_MSTR);
				ST_GiveForcePower(JD_PROTECT_MSTR);
				break;				
		}
	}
}


// ---------------------------------------------------------------
// Returns TRUE if oTarget has a lightsaber equipped.
// ---------------------------------------------------------------
int ST_GetHasLightsaber(object oTarget=OBJECT_SELF) {
	object oItem = GetItemInSlot(INVENTORY_SLOT_RIGHTWEAPON, oTarget);
	
	if (!GetIsObjectValid(oItem))
		return FALSE;
	
	int nBase = GetBaseItemType(oItem);
	
	return ((nBase == BASE_ITEM_LIGHTSABER)	
		|| (nBase == BASE_ITEM_DOUBLE_BLADED_LIGHTSABER)
		|| (nBase == BASE_ITEM_SHORT_LIGHTSABER));
}


// ---------------------------------------------------------------
// Returns TRUE if the specified Feat ID matches that of an active
// Melee combat feat.
// ---------------------------------------------------------------
int ST_GetIsCombatFeat(int nFeat) {
	switch (nFeat) {
		case FEAT_CRITICAL_STRIKE: 
		case FEAT_IMPROVED_CRITICAL_STRIKE: 
		case FEAT_MASTER_CRITICAL_STRIKE: 
		case FEAT_FLURRY: 
		case FEAT_IMPROVED_FLURRY: 
		case FEAT_WHIRLWIND_ATTACK: 
		case FEAT_POWER_ATTACK: 
		case FEAT_IMPROVED_POWER_ATTACK: 
		case FEAT_MASTER_POWER_ATTACK: return TRUE;
	}
	return FALSE;
}

// ---------------------------------------------------------------
// Wrapper function for ActionUseTalentOnObject(), allowing it to
// apply some talent filters and checks before a talent is used.
// ---------------------------------------------------------------
int ST_ActionUseTalent(talent tUse, object oTarget, int bSetTarget=TRUE, int bClearActions=TRUE) {
	int nType = GetTypeFromTalent(tUse);
	int nId   = GetIdFromTalent(tUse);	
	
	// If the target is not valid there is no point in continuing.
	if (!GetIsObjectValid(oTarget)) 
		return FALSE;
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Do special checks and substitutions for usable feats....
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	if (nType == TALENT_TYPE_FEAT) {
		tUse = ST_GetCorrectCombatFeat(tUse);
		
		// These feats are not usable. Skip them if they ever appear here.
		if ((nId == 1) || (nId == 2) || (nId == 24) || (nId == 25) || (nId == 54) || (nId == 55))
			return FALSE;
			
		// The Wookiee Rage feats can't be used by the AI for some reason, they'll
		// just stand around like idiots doing nothing. use the workaround...
		if ((nId == 231) || (nId == 232) || (nId == 233))
			return ST_RageFeatHack(nId);
		
		// ADDED(2005-09-17)
		// Substitute combat feat use with Default Attack if the target is viable
		// for performing a Force Jump Attack instead.	
		if ((GetHasFeat(101) || GetHasFeat(102) || GetHasFeat(103))
			&& ST_GetIsCombatFeat(nId) 
			&& ST_GetHasLightsaber(OBJECT_SELF)
			&& (GetDistanceBetween2D(oTarget, OBJECT_SELF) > 10.1)
			&& ST_GetHasClearPathTo(oTarget))
		{
			if (bClearActions)
				ClearAllActions();
			
			ActionAttack(oTarget);
				
			if (bSetTarget)
				ST_SetLastAttackTarget(oTarget);
				
			return TRUE;					
		}
	}
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Do special checks and substitutions for force powers
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	else if (nType == TALENT_TYPE_FORCE) {
		tUse = ST_GetCorrectForcePower(tUse);
		
		// Filter force powers which are unsuitable to use against this target
		if ((oTarget != OBJECT_SELF) && !ST_ShouldUseDisablingPower(tUse, oTarget))
			return FALSE;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Filters and substitutions done, use talent if still valid...
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	if (GetIsTalentValid(tUse) && GetCreatureHasTalent(tUse)) {
		if (bClearActions)
			ClearAllActions();
			
		ActionUseTalentOnObject(tUse, oTarget);
		
		if (bSetTarget)
			ST_SetLastAttackTarget(oTarget);
		
		return TRUE;
	}
	return FALSE;
}


// ---------------------------------------------------------------
// Hackish support function, if the type id specified as a 
// parameter, which should be a FEAT.2DA index, is one of the
// Wookiee rage feats, cast the associated "spell" directly 
// instead since the feats can't be used by the AI for some 
// reason. They just stand there doing nothing when trying.
// ---------------------------------------------------------------
int ST_RageFeatHack(int nType) {
	switch (nType) {
		case 233:  
			ClearAllActions();
			ActionCastSpellAtObject(FORCE_POWER_WOOKIEE_RAGE_III, OBJECT_SELF, 0 , TRUE); 
			return TRUE;
			
		case 232:  
			ClearAllActions();
			ActionCastSpellAtObject(FORCE_POWER_WOOKIEE_RAGE_II, OBJECT_SELF, 0 , TRUE);  
			return TRUE;
			
		case 231:  
			ClearAllActions();
			ActionCastSpellAtObject(FORCE_POWER_WOOKIEE_RAGE_I, OBJECT_SELF, 0 , TRUE); 	 
			return TRUE;
	}	
	return FALSE;
}


// ---------------------------------------------------------------
// Use Wookiee Rage if the creature knows it and is not already
// enraged.
//
// WTF: Apparently NPC AI can't activate the Rage feats, they'll
//      just stand there doing absolutely NOTHING. Lovely.
//      EDIT: Moved the workaround for this to the Talent filter.
// ---------------------------------------------------------------
int ST_TalentUseRage() {
    if (!ST_GetIsEnraged(OBJECT_SELF)
    	&& !GetLocalBoolean(OBJECT_SELF, 66) // CONDITION: SW_FLAG_FAST_BUFF
		&& (GetCurrentAction(OBJECT_SELF) != ACTION_CASTSPELL)
		&& (GetCurrentAction(OBJECT_SELF) != ACTION_ITEMCASTSPELL))  
    {
    	talent tRage;
    	int i;
    	int nMax = 6;
    	
    	if (ST_GetIsForceUser())
    		nMax = 9;
    	
    	for (i = 0; i < nMax; i++) {
			switch (i) {
				case 0: tRage = TalentSpell(FORCE_POWER_WOOKIEE_RAGE_III); break;
				case 1: tRage = TalentFeat(233);  break;
				case 2: tRage = TalentSpell(FORCE_POWER_WOOKIEE_RAGE_II); break;
				case 3: tRage = TalentFeat(232);  break;
				case 4:	tRage = TalentSpell(FORCE_POWER_WOOKIEE_RAGE_I); break;
				case 5: tRage = TalentFeat(231);  break;
				case 6:	tRage = TalentSpell(166); break;
				case 7:	tRage = TalentSpell(165); break;
				case 8:	tRage = TalentSpell(164); break;
			}    		
        	
        	if (GetIsTalentValid(tRage) && GetCreatureHasTalent(tRage)) {
        		if (ST_ActionUseTalent(tRage, OBJECT_SELF, FALSE)) {
        			SetLocalBoolean(OBJECT_SELF, 66, TRUE);
        			DelayCommand(18.0, SetLocalBoolean(OBJECT_SELF, 66, FALSE));        		
        			return TRUE;	
        		}
        	} 
        }
    }

    return FALSE;
}


// ---------------------------------------------------------------
// Wrapper function for use in the AI_MASTER script to check that
// an object isn't just valid, but that they aren't already dead
// and that they can be seen to qualify as a valid target.
// ---------------------------------------------------------------
int ST_GetIsAITargetValid(object oTarget) {
	return (GetIsObjectValid(oTarget) 
		&& !GetIsDead(oTarget) 
		&& (GetCurrentHitPoints(oTarget) > 0)
		&& (GetObjectSeen(oTarget) 
			|| (IsObjectPartyMember(oTarget) && GetObjectSeen(GetPartyLeader(), oTarget))
			|| (GetAttemptedMovementTarget() == oTarget)));
}


// ---------------------------------------------------------------
// Simple wrapper to get the nearest seen enemy of a creature.
// ---------------------------------------------------------------
object ST_GetNearestEnemy(object oSource = OBJECT_SELF) {
	int nIdx = 1;
	object oEnemy = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, oSource, nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);	
	
	while (GetIsObjectValid(oEnemy) && (GetIsDead(oEnemy) || (GetCurrentHitPoints(oEnemy) < 1)) )
		oEnemy = GetNearestCreature(CREATURE_TYPE_REPUTATION, REPUTATION_TYPE_ENEMY, oSource, ++nIdx, CREATURE_TYPE_PERCEPTION, PERCEPTION_SEEN);	
		
	if (GetIsDead(oEnemy) || (GetCurrentHitPoints(oEnemy) < 1)) 
		return OBJECT_INVALID;
		
	return oEnemy;
}


// ---------------------------------------------------------------
// Returns TRUE if oTarget is unable to attack due to being
// affected by some incapacitating effect. Far from perfect since
// they for some reason haven't added effect types for CHOKE, 
// PUSHED, WHIRLWIND and KNOCKDOWN, making them virtually 
// undetectable.
// ---------------------------------------------------------------
int ST_GetIsHelpless(object oTarget) {
    effect eEff = GetFirstEffect(oTarget);
    int nType;
    
    while (GetIsEffectValid(eEff)) {
        nType = GetEffectType(eEff);
        if (   (nType == EFFECT_TYPE_PARALYZE)
            || (nType == EFFECT_TYPE_STUNNED)
            || (nType == EFFECT_TYPE_FRIGHTENED)
            || (nType == EFFECT_TYPE_SLEEP)
            || (nType == EFFECT_TYPE_CONFUSED)
            || (nType == EFFECT_TYPE_DROID_CONFUSED)
            || (nType == EFFECT_TYPE_DROIDSCRAMBLE)
            || (nType == EFFECT_TYPE_MINDTRICK))
        {
            return TRUE;
        }
        eEff = GetNextEffect(oTarget);
    }
    return FALSE;
}


// ---------------------------------------------------------------
// Support function for ST_GetRandomProjectileTalent().
// nNum = Iterator for getting the next number in sequence.
// nSet = The sequence to use, randomly picked.
// ---------------------------------------------------------------
int ST_GetNextProjectileNumber(int nNum, int nSet) {
	int nRes = 0;
	switch (nSet) {
		case 0: 
			switch (nNum) {
				case 0: nRes = 190; break;
				case 1: nRes = 194; break;
				case 2: nRes = 195; break;
				case 3: nRes = 196; break;
				case 4: nRes = 197; break;
				case 5: nRes = 198; break;
				case 6: nRes = 199; break;
			}
			break;
		
		case 1: 
			switch (nNum) {
				case 0: nRes = 199; break;
				case 1: nRes = 190; break;
				case 2: nRes = 194; break;
				case 3: nRes = 195; break;
				case 4: nRes = 196; break;
				case 5: nRes = 197; break;
				case 6: nRes = 198; break;
			}
			break;		
		case 2: 
			switch (nNum) {
				case 0: nRes = 198; break;
				case 1: nRes = 199; break;
				case 2: nRes = 190; break;
				case 3: nRes = 194; break;
				case 4: nRes = 195; break;
				case 5: nRes = 196; break;
				case 6: nRes = 197; break;
			}
			break;		
		case 3: 
			switch (nNum) {
				case 0: nRes = 197; break;
				case 1: nRes = 198; break;
				case 2: nRes = 199; break;
				case 3: nRes = 190; break;
				case 4: nRes = 194; break;
				case 5: nRes = 195; break;
				case 6: nRes = 196; break;
			}
			break;		
		case 4: 
			switch (nNum) {
				case 0: nRes = 196; break;
				case 1: nRes = 197; break;
				case 2: nRes = 198; break;
				case 3: nRes = 199; break;
				case 4: nRes = 190; break;
				case 5: nRes = 194; break;
				case 6: nRes = 195; break;
			}
			break;		
		case 5: 
			switch (nNum) {
				case 0: nRes = 195; break;
				case 1: nRes = 196; break;
				case 2: nRes = 197; break;
				case 3: nRes = 198; break;
				case 4: nRes = 199; break;
				case 5: nRes = 190; break;
				case 6: nRes = 194; break;
			}
			break;		
		case 6:	
			switch (nNum) {
				case 0: nRes = 194; break;
				case 1: nRes = 195; break;
				case 2: nRes = 196; break;
				case 3: nRes = 197; break;
				case 4: nRes = 198; break;
				case 5: nRes = 199; break;
				case 6: nRes = 190; break;
			}
			break;			
	}
	return nRes;
}


// ---------------------------------------------------------------
// Returns a randomly picked rocketlauncher fired Projectile 
// talent, which does not have any blast radius.
// ---------------------------------------------------------------
talent ST_GetRandomProjectileTalent() {
	talent tCheck;
	talent tInvalid;
	int nSet = Random(7);
	int nNum;
	int nProj;

	for (nNum = 0; nNum < 7; nProj = ST_GetNextProjectileNumber(nNum++, nSet)) {
		tCheck = TalentSpell(nProj);
		if (GetIsTalentValid(tCheck) && GetCreatureHasTalent(tCheck)) {
			return tCheck;
		}	
	}
	return tInvalid;				
}


// ---------------------------------------------------------------
// If the creature running this is a force user of high enough
// level and armed with a lightsaber, they will be assigned and
// set to use a force form.
// ---------------------------------------------------------------
void ST_DoSaberFormUsage() {
	if (GetLocalBoolean(OBJECT_SELF, 88) || IsObjectPartyMember(OBJECT_SELF))
		return;
	
	SetLocalBoolean(OBJECT_SELF, 88, TRUE);
	
	if (ST_GetIsForceUser() && ST_GetHasLightsaber()) {
		int nForm;
		
		// First check if the creature already knows any forms. If so, use that one.
		for (nForm = FORM_FORCE_IV_MASTERY; nForm >= FORM_SABER_I_SHII_CHO; nForm--) {
			if (GetSpellAcquired(nForm) && !IsFormActive(OBJECT_SELF, nForm)) {
				SetCurrentForm(OBJECT_SELF, nForm);		
				return;	
			}			
		}
		
		// No forms known, pick a suitable one, grant them it and make them use it.
		int bBoss =  GetLocalBoolean(OBJECT_SELF, 89);
		int nLvl  = GetHitDice(OBJECT_SELF);		
		nForm     = 0;

		// Only use Form 1 if lvl 12 or above but below lvl 15....
		if (!bBoss && (nLvl > 11) && (nLvl < 15)) {
			nForm = FORM_SABER_I_SHII_CHO;		
		}			  
		else if (GetLevelByClass(CLASS_TYPE_JEDIWEAPONMASTER) || GetLevelByClass(CLASS_TYPE_SITHMARAUDER)) {
			nForm = FORM_SABER_VII_JUYO;
		}
		else if (GetLevelByClass(CLASS_TYPE_JEDIWATCHMAN) || GetLevelByClass(CLASS_TYPE_SITHASSASSIN)) {
			nForm = FORM_SABER_VI_NIMAN;
		}		
		else if (GetLevelByClass(CLASS_TYPE_JEDIMASTER) || GetLevelByClass(CLASS_TYPE_SITHLORD)) {
			if (nLvl < 20)
				nForm = FORM_FORCE_III_AFFINITY;
			else if (nLvl < 30)
				nForm = FORM_FORCE_II_POTENCY;
			else
				nForm = FORM_FORCE_IV_MASTERY;
		}
		else {
			if (GetLevelByClass(CLASS_TYPE_JEDICONSULAR)) {
				if (nLvl < 25)
					nForm = FORM_FORCE_I_FOCUS;	
				else
					nForm = FORM_FORCE_III_AFFINITY;
			}
			else if (GetLevelByClass(CLASS_TYPE_JEDISENTINEL)) {
				if (nLvl < 18) {
					if (Random(100) < 50)
						nForm = FORM_SABER_I_SHII_CHO;
					else
						nForm = FORM_FORCE_I_FOCUS;	
				}
				else {
					switch (Random(5)) {
						case 0: nForm = FORM_SABER_IV_ATARU;   break;
						case 1: nForm = FORM_SABER_V_SHIEN;    break;
						case 2: nForm = FORM_SABER_III_SORESU; break;
						case 3: nForm = FORM_SABER_I_SHII_CHO; break;
						case 4: nForm = FORM_FORCE_I_FOCUS;    break;
					}	
				}
			}
			if (GetLevelByClass(CLASS_TYPE_JEDIGUARDIAN)) {
				if (nLvl < 16) {
					if (Random(100) < 50)
						nForm = FORM_SABER_I_SHII_CHO;
					else
						nForm = FORM_SABER_II_MAKASHI;
						
				}
				else {
					switch (Random(5)) {
						case 0: nForm = FORM_SABER_IV_ATARU;   break;
						case 1: nForm = FORM_SABER_V_SHIEN;    break;
						case 2: nForm = FORM_SABER_III_SORESU; break;
						case 3: nForm = FORM_SABER_I_SHII_CHO; break;
						case 4: nForm = FORM_SABER_II_MAKASHI; break;
					}	
				}
			}			
		}
		
		if (nForm != 0) {
			if (!GetSpellAcquired(nForm))
				GrantSpell(nForm, OBJECT_SELF);
				
			if (!IsFormActive(OBJECT_SELF, nForm))
				SetCurrentForm(OBJECT_SELF, nForm);	
		}				
	}
}


// ---------------------------------------------------------------
// Checks if oTarget already is affected by the performance
// enhancing drug set by nSpell, or one of similar effect.
// ---------------------------------------------------------------
int ST_GetHasStimEffect(int nSpell, object oTarget=OBJECT_SELF) {
	// Combat Shot stims...
	if (((nSpell == 77) 
			|| (nSpell == 76)  
			|| (nSpell == 75))
		&& (GetHasSpellEffect(77, oTarget) 
			|| GetHasSpellEffect(76, oTarget) 
			|| GetHasSpellEffect(75, oTarget)))
	{
		return TRUE;
	}
	
	// Stamina Stims....
	if (((nSpell == 74) || (nSpell == 71)) && (GetHasSpellEffect(74, oTarget) || GetHasSpellEffect(71, oTarget))) {
		return TRUE;	
	}
	
	// Alacrity Stims....
	if (((nSpell == 73) || (nSpell == 70)) && (GetHasSpellEffect(73, oTarget) || GetHasSpellEffect(70, oTarget))) {
		return TRUE;	
	}
	
	// Strength Stims....
	if (((nSpell == 72) || (nSpell == 69)) && (GetHasSpellEffect(72, oTarget) || GetHasSpellEffect(69, oTarget))) {
		return TRUE;	
	}	
	
	return FALSE;
}


// ---------------------------------------------------------------
// Makes oUser use any performance enhancing drugs in their
// inventory to buff up during battle, unless it's a droid.
// ---------------------------------------------------------------
int ST_TalentBuffUseStims(object oUser=OBJECT_SELF) {
	// Don't use several stims in a row without doing anything inbetween.
	if (GetLocalBoolean(oUser, 67)) 
		return FALSE;	
		
	// Droids can't use stims...
	if (GetRacialType(oUser) == RACIAL_TYPE_DROID)
		return FALSE;
	
	// Partymembers shouldn't automatically buff with consumables.	
	if (IsObjectPartyMember(oUser)) 
		return FALSE;	
	
	int i;
	talent tStim;
	
	for (i = 77; i >= 69; i--) {
		tStim = TalentSpell(i);
		
		if (GetIsTalentValid(tStim) 
			&& GetCreatureHasTalent(tStim, oUser)
			&& !ST_GetHasStimEffect(i, oUser))
		{ 	
            if (ST_ActionUseTalent(tStim, oUser, FALSE)) {
            	SetLocalBoolean(oUser, 67, TRUE);
	        	DelayCommand(7.0, SetLocalBoolean(oUser, 67, FALSE));	
	        	return TRUE;	
	        }
		}
	}
	
	return FALSE;
}


// ---------------------------------------------------------------
// Ad-hoc support for the "SPIRIT" feats when added to the main
// character. 
// ---------------------------------------------------------------
void ST_CheckRevivePlayer() { 
    // Only make party members do this.
    if (!IsObjectPartyMember(OBJECT_SELF))
        return;
    
    object oTarget = GetFirstPC();
    
    // Abort if the main character could not be located yet.     
    if(!GetIsObjectValid(oTarget))
        return;
        
    // The main character shouldn't run this on themselves...
    if (oTarget == OBJECT_SELF)
    	return;            
      
    // If I'm dead myself, no point doing this...
    if ((GetCurrentHitPoints(OBJECT_SELF) < 1) || GetIsDead(OBJECT_SELF))
        return;

    // If the main character isn't dead, no point continuing...
    if ((GetCurrentHitPoints(oTarget) > 0) && !GetIsDead(oTarget))
        return;

    // Only proceed if the main character has one of the feats...
    if (   !GetHasFeat(FEAT_SPIRIT, oTarget) 
        && !GetHasFeat(FEAT_FIGHTING_SPIRIT, oTarget) 
        && !GetHasFeat(FEAT_HEROIC_RESOLVE, oTarget)) 
    {
        return;
    }

    // Check to see if any other party member is still alive.
    int bPartyNotDead = FALSE;
    int nIdx = 0;
    object oParty = GetPartyMemberByIndex(nIdx);
    while (GetIsObjectValid(oParty)) {
        if ((GetCurrentHitPoints(oParty) > 0) 
            && !GetIsDead(oParty)
            && GetIsInCombat(oParty, TRUE)) 
        {
            bPartyNotDead = TRUE;
            break;
        }
        oParty = GetPartyMemberByIndex(++nIdx);
    }

    // Someone was alive, attempt to revive the main character.
    if (bPartyNotDead) {
        if (GetCurrentHitPoints(oTarget) < 1) {
            int nHPPercent = 0;
            if( GetHasFeat(FEAT_HEROIC_RESOLVE, oTarget) )
                nHPPercent = 30;
            else if( GetHasFeat(FEAT_FIGHTING_SPIRIT, oTarget) )
                nHPPercent = 20;
            else if( GetHasFeat(FEAT_SPIRIT, oTarget) )
                nHPPercent = 10;

            ApplyEffectToObject(DURATION_TYPE_INSTANT, EffectResurrection(nHPPercent), oTarget);
            ClearAllActions();
        }

        // Kickstart combat AI again if not player controlled...
        if (!GetIsPartyLeader(oTarget))
            DelayCommand(4.0, ExecuteScript("k_ai_master", oTarget, 2003));
    }
}


// ------------------------------------------------------------------
// ST: 2006-07-17 - Check if no weapons but a shock arm is equipped.
// ------------------------------------------------------------------
int ST_HasOnlyDroidShockArm() {
	return (!GetIsObjectValid(GetItemInSlot(INVENTORY_SLOT_RIGHTWEAPON))
	&& (GetTag(GetItemInSlot(INVENTORY_SLOT_RIGHTARM)) == "d_t3m4_01") 
	|| (GetTag(GetItemInSlot(INVENTORY_SLOT_LEFTARM)) == "d_t3m4_01"));
}