/* R.Taylor 12/18/03
 * This script is called from the OnDeath event on Atton. It
 * is responsible for handling the effects of the Spirit Feat, which
 * make him get up when dropped in battle, as long as there is still
 * at least 1 other standing party member.
 */
#include "jd_constants"
#include "k_inc_switch"

void DoRevive(string sTarget)
{
    object oTarget = GetObjectByTag( sTarget );
    if( !GetIsObjectValid( oTarget ) ) {
        return;
    }

    AurPostString( "Perform Revive.", 5, 6, 10.0 );
    if ( !GetHasFeat(FEAT_SPIRIT, oTarget) &&
         !GetHasFeat(FEAT_FIGHTING_SPIRIT, oTarget) &&
         !GetHasFeat(FEAT_HEROIC_RESOLVE, oTarget) ) {
        AurPostString( "No Spirit.", 5, 7, 10.0 );
        return;//Cancel out if doesn't have spirit feat
    }

    //First determine if we have at least 1 standing ally.
    int bPartyNotDead = 0;
    int nIdx = 0;
    object oParty = GetPartyMemberByIndex(nIdx);
    AurPostString( "Got first party member.", 5, 7, 10.0 );
    while (GetIsObjectValid(oParty))
    {
        //RWT-OEI 09/30/04 - The member has to be standing AND in combat
        //to get Atton to revive via this script.
        if ( GetCurrentHitPoints(oParty) > 0 && GetIsInCombat(oParty, TRUE) ) {//1 is still standing
            AurPostString( "Found a valid party member.", 5, 8+nIdx, 10.0 );
            bPartyNotDead = 1;
            break;
        }
        nIdx++;
        AurPostString( "Getting next party member.", 8, 8+nIdx, 10.0 );
        oParty = GetPartyMemberByIndex(nIdx);
    }

    if ( bPartyNotDead ) {
        AurPostString( "Party not dead 2.", 5, 7, 10.0 );
        if ( GetCurrentHitPoints(oTarget) < 1 ) {
            AurPostString( "HP less than 1.", 5, 8, 10.0 );
            // DJS-OEI 8/26/2004
            int nHPPercent = 0;
            if( GetHasFeat(FEAT_HEROIC_RESOLVE, oTarget) ) {
                nHPPercent = 30;
            }
            else if( GetHasFeat(FEAT_FIGHTING_SPIRIT, oTarget) ) {
                nHPPercent = 20;
            }
            else if( GetHasFeat(FEAT_SPIRIT, oTarget) ) {
                nHPPercent = 10;
            }

            ApplyEffectToObject(DURATION_TYPE_INSTANT, EffectResurrection( nHPPercent ),
                                oTarget);
            ClearAllActions();//Clear our action queue as we get up.
        }
        else {
            AurPostString( "HP 1 or greater.", 5, 8, 10.0 );
        }
        //Need to get the character back into combat after getting up.
        //The delay is to give him enough time to animate to standing before
        //jumping back into the battle
        DelayCommand(4.0,
            ExecuteScript("k_ai_master", oTarget, KOTOR_HENCH_EVENT_ON_COMBAT_ROUND_END));
    }
    else
    {
        AurPostString( "Party dead.", 5, 7, 10.0 );
        //If we aren't getting revived right now, try again in 5 seconds
        //DelayCommand( 5.0f, DoRevive(myself));
        object oArea = GetArea( oTarget );
        AssignCommand( oArea, DelayCommand( 5.0f, DoRevive(sTarget)));
    }
}

void main()
{
    //RWT-OEI 09/30/04 - Only queue if we're down, this way we can check this
    //function from Atton's heartbeat.
//    if ( GetCurrentHitPoints(OBJECT_SELF) )
//    {
        // DJS-OEI
        // Assigned command to the area so it will work when game is loaded.
        // This is really stupid, but passing OBJECT_SELF to DoRevive()
        // will pass the area's ID instead of this guy's.
        string sName = GetTag( OBJECT_SELF );
        object oArea = GetArea( OBJECT_SELF );
        AssignCommand( oArea, DelayCommand( 8.0f, DoRevive(sName)));
//    }
    int nRelatedPowerID1 = JD_REZ_SUP_MSTR;
    int nRelatedPowerID2 = JD_DEMO;
    int nRelatedPowerID3 = JD_IMP_DEMO;
    int nRelatedPowerID4 = JD_MSTR_DEMO;
    int nRelatedPowerID5 = JD_AWARE;
    int nRelatedPowerID6 = JD_IMP_AWARE;
    int nRelatedPowerID7 = JD_MSTR_AWARE;
    int nRelatedPowerID8 = JD_COMP;
    int nRelatedPowerID9 = JD_IMP_COMP;
    int nRelatedPowerID10 = JD_MSTR_COMP;
    int nRelatedPowerID11 = JD_PERS;
    int nRelatedPowerID12 = JD_IMP_PERS;
    int nRelatedPowerID13 = JD_MSTR_PERS;
    int nRelatedPowerID14 = JD_REPAIR;
    int nRelatedPowerID15 = JD_IMP_REPAIR;
    int nRelatedPowerID16 = JD_MSTR_REPAIR;
    int nRelatedPowerID17 = JD_SECURITY;
    int nRelatedPowerID18 = JD_IMP_SECURITY;
    int nRelatedPowerID19 = JD_MSTR_SECURITY;
    int nRelatedPowerID20 = JD_STEALTH;
    int nRelatedPowerID21 = JD_IMP_STEALTH;
    int nRelatedPowerID22 = JD_MSTR_STEALTH;
    int nRelatedPowerID23 = JD_TREAT_INJURY;
    int nRelatedPowerID24 = JD_IMP_TREAT_INJURY;
    int nRelatedPowerID25 = JD_MSTR_TREAT_INJURY;
    int nRelatedPowerID26 = FORCE_POWER_SPEED_MASTERY;
    int nRelatedPowerID27 = FORCE_POWER_KNIGHT_SPEED;
    int nRelatedPowerID28 = FORCE_POWER_SPEED_BURST; 
    int nRelatedPowerID29 = FORCE_POWER_FORCE_ARMOR; 
    int nRelatedPowerID30 = FORCE_POWER_FORCE_SHIELD; 
    int nRelatedPowerID31 = FORCE_POWER_FORCE_AURA; 
    int nRelatedPowerID32 = FORCE_POWER_MIND_MASTERY; 
    int nRelatedPowerID33 = FORCE_POWER_KNIGHT_MIND; 
    int nRelatedPowerID34 = FORCE_POWER_FORCE_MIND; 
    int nRelatedPowerID35 = FORCE_POWER_MASTER_ENERGY_RESISTANCE; 
    int nRelatedPowerID36 = FORCE_POWER_RESIST_COLD_HEAT_ENERGY; 
    int nRelatedPowerID37 = FORCE_POWER_RESIST_POISON_DISEASE_SONIC; 
    int nRelatedPowerID38 = FORCE_POWER_MASTER_FORCE_BARRIER; 
    int nRelatedPowerID39 = FORCE_POWER_IMPROVED_FORCE_BARRIER; 
    int nRelatedPowerID40 = FORCE_POWER_FORCE_BARRIER; 
    int nRelatedPowerID41 = FORCE_POWER_MASTER_BATTLE_MEDITATION_PC; 
    int nRelatedPowerID42 = FORCE_POWER_IMPROVED_BATTLE_MEDITATION_PC; 
    int nRelatedPowerID43 = FORCE_POWER_BATTLE_MEDITATION_PC; 
    int nRelatedPowerID44 = FORCE_POWER_FORCE_IMMUNITY; 
    int nRelatedPowerID45 = FORCE_POWER_RESIST_FORCE; 
    int nRelatedPowerID46 = FORCE_POWER_INSPIRE_FOLLOWERS_VI; 
    int nRelatedPowerID47 = FORCE_POWER_INSPIRE_FOLLOWERS_V; 
    int nRelatedPowerID48 = FORCE_POWER_INSPIRE_FOLLOWERS_IV; 
    int nRelatedPowerID49 = FORCE_POWER_INSPIRE_FOLLOWERS_III; 
    int nRelatedPowerID50 = FORCE_POWER_INSPIRE_FOLLOWERS_II; 
    int nRelatedPowerID51 = FORCE_POWER_INSPIRE_FOLLOWERS_I;
    int nRelatedPowerID52 = JD_RECK;
    int nRelatedPowerID53 = JD_RECK_IMP;
    int nRelatedPowerID54 = JD_RECK_MSTR;
    int nRelatedPowerID55 = JD_RAGE;
    int nRelatedPowerID56 = JD_RAGE_IMP;
    int nRelatedPowerID57 = JD_RAGE_MSTR;
    int nRelatedPowerID58 = JD_DECEIVE;
    int nRelatedPowerID59 = JD_DECEIVE_IMP;
    int nRelatedPowerID60 = JD_DECEIVE_MSTR;
    int nRelatedPowerID61 = JD_ENERGY;
    int nRelatedPowerID62 = JD_ENERGY_IMP;
    int nRelatedPowerID63 = JD_ENERGY_MSTR;
    int nRelatedPowerID64 = JD_REGEN;
    int nRelatedPowerID65 = JD_REGEN_IMP;
    int nRelatedPowerID66 = JD_REGEN_MSTR;
    int nRelatedPowerID67 = JD_PROTECT;
    int nRelatedPowerID68 = JD_PROTECT_IMP;
    int nRelatedPowerID69 = JD_PROTECT_MSTR;
    int nRelatedPowerID70 = JD_OMNI;
    int nRelatedPowerID71 = JD_OMNI_IMP;
    int nRelatedPowerID72 = JD_OMNI_MSTR;
    int nRelatedPowerID73 = JD_PREP;
    int nRelatedPowerID74 = JD_PREP_IMP;
    int nRelatedPowerID75 = JD_PREP_MSTR;
    int nRelatedPowerID76 = JD_PREP_SUP;
    int nRelatedPowerID77 = JD_PREP_SUP_IMP;
    int nRelatedPowerID78 = JD_PREP_SUP_MSTR;
    if  ( GetHasSpellEffect(nRelatedPowerID1, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID2, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID3, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID4, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID5, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID6, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID7, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID8, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID9, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID10, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID11, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID12, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID13, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID14, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID15, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID16, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID17, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID18, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID19, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID20, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID21, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID22, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID23, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID24, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID25, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID26, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID27, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID28, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID29, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID30, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID31, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID32, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID33, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID34, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID35, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID36, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID37, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID38, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID39, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID40, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID41, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID42, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID43, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID44, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID45, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID46, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID47, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID48, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID49, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID50, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID51, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID52, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID53, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID54, OBJECT_SELF) || 
        GetHasSpellEffect(nRelatedPowerID55, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID56, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID57, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID58, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID59, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID60, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID61, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID62, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID63, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID64, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID65, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID66, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID67, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID68, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID69, OBJECT_SELF) || 
        GetHasSpellEffect(nRelatedPowerID70, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID71, OBJECT_SELF) ||        
        GetHasSpellEffect(nRelatedPowerID72, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID73, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID74, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID75, OBJECT_SELF) || 
        GetHasSpellEffect(nRelatedPowerID76, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID77, OBJECT_SELF) ||
        GetHasSpellEffect(nRelatedPowerID78, OBJECT_SELF) )
        {
            effect eRelatedEffect = GetFirstEffect(OBJECT_SELF);
            while (GetIsEffectValid(eRelatedEffect))
            {
            if( 	GetEffectSpellId(eRelatedEffect) == nRelatedPowerID1 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID2 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID3 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID4 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID5 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID6 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID7 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID8 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID9 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID10 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID11 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID12 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID13 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID14 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID15 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID16 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID17 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID18 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID19 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID20 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID21 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID22 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID23 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID24 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID25 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID26 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID27 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID28 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID29 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID30 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID31 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID32 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID33 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID34 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID35 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID36 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID37 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID38 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID39 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID40 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID41 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID42 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID43 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID44 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID45 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID46 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID47 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID48 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID49 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID50 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID51 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID52 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID53 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID54 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID55 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID56 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID57 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID58 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID59 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID60 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID61 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID62 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID63 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID64 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID65 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID66 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID67 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID68 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID69 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID70 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID71 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID72 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID73 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID74 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID75 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID76 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID77 ||
                GetEffectSpellId(eRelatedEffect) == nRelatedPowerID78 )
                {
                    RemoveEffect(OBJECT_SELF, eRelatedEffect);
                }
                eRelatedEffect = GetNextEffect(OBJECT_SELF);
              }
        }
}
